@@ -2,7 +2,7 @@
  * input_ext-dec.h: structures exported to the VideoLAN decoders
  *****************************************************************************
  * Copyright (C) 1999-2001 VideoLAN
- * $Id: input_ext-dec.h,v 1.59 2002/05/24 12:42:14 gbazin Exp $
+ * $Id: input_ext-dec.h,v 1.79 2003/03/04 13:21:19 massiot Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *          Michel Kaempf <maxx@via.ecp.fr>
@@ -22,21 +22,8 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  *****************************************************************************/
 
-/* ES streams types - see ISO/IEC 13818-1 table 2-29 numbers */
-#define MPEG1_VIDEO_ES      0x01
-#define MPEG2_VIDEO_ES      0x02
-#define MPEG1_AUDIO_ES      0x03
-#define MPEG2_AUDIO_ES      0x04
-#define AC3_AUDIO_ES        0x81
-/* These ones might violate the norm : */
-#define DVD_SPU_ES          0x82
-#define LPCM_AUDIO_ES       0x83
-#define MSMPEG4v1_VIDEO_ES  0x40
-#define MSMPEG4v2_VIDEO_ES  0x41
-#define MSMPEG4v3_VIDEO_ES  0x42
-#define MPEG4_VIDEO_ES      0x50
-
-#define UNKNOWN_ES          0xFF
+#ifndef _VLC_INPUT_EXT_DEC_H
+#define _VLC_INPUT_EXT_DEC_H 1
 
 /* Structures exported to the decoders */
 
@@ -45,22 +32,22 @@
  *****************************************************************************
  * Describe a data packet.
  *****************************************************************************/
-typedef struct data_packet_s
+struct data_packet_t
 {
     /* Used to chain the packets that carry data for a same PES or PSI */
-    struct data_packet_s *  p_next;
+    data_packet_t *  p_next;
 
     /* start of the PS or TS packet */
-    byte_t *                p_demux_start;
+    byte_t *         p_demux_start;
     /* start of the PES payload in this packet */
-    byte_t *                p_payload_start;
-    byte_t *                p_payload_end; /* guess ? :-) */
+    byte_t *         p_payload_start;
+    byte_t *         p_payload_end; /* guess ? :-) */
     /* is the packet messed up ? */
-    boolean_t               b_discard_payload;
+    vlc_bool_t       b_discard_payload;
 
     /* pointer to the real data */
-    struct data_buffer_s *  p_buffer;
-} data_packet_t;
+    data_buffer_t *  p_buffer;
+};
 
 /*****************************************************************************
  * pes_packet_t
@@ -68,55 +55,65 @@ typedef struct data_packet_s
  * Describes an PES packet, with its properties, and pointers to the TS packets
  * containing it.
  *****************************************************************************/
-typedef struct pes_packet_s
+struct pes_packet_t
 {
     /* Chained list to the next PES packet (depending on the context) */
-    struct pes_packet_s *   p_next;
+    pes_packet_t *  p_next;
 
     /* PES properties */
-    boolean_t               b_data_alignment;  /* used to find the beginning of
-                                                * a video or audio unit      */
-    boolean_t               b_discontinuity; /* This packet doesn't follow the
-                                              * previous one                 */
+    vlc_bool_t      b_data_alignment;          /* used to find the beginning of
+                                                * a video or audio unit */
+    vlc_bool_t      b_discontinuity;          /* This packet doesn't follow the
+                                               * previous one */
 
-    mtime_t                 i_pts;    /* PTS for this packet (zero if unset) */
-    mtime_t                 i_dts;    /* DTS for this packet (zero if unset) */
-    int                     i_rate;                /* current pace of reading
-                                                    * (see stream_control.h) */
+    mtime_t         i_pts;            /* PTS for this packet (zero if unset) */
+    mtime_t         i_dts;            /* DTS for this packet (zero if unset) */
+    int             i_rate;   /* current reading pace (see stream_control.h) */
 
-    unsigned int            i_pes_size;    /* size of the current PES packet */
+    unsigned int    i_pes_size;            /* size of the current PES packet */
 
     /* Chained list to packets */
-    data_packet_t *         p_first;      /* The first packet contained by this
+    data_packet_t * p_first;              /* The first packet contained by this
                                            * PES (used by decoders). */
-    data_packet_t *         p_last;    /* The last packet contained by this
-                                          PES (used by the buffer allocator) */
-    unsigned int            i_nb_data; /* Number of data packets in the chained
-                                                                        list */
-} pes_packet_t;
+    data_packet_t * p_last;            /* The last packet contained by this
+                                        * PES (used by the buffer allocator) */
+    unsigned int    i_nb_data; /* Number of data packets in the chained list */
+};
 
 /*****************************************************************************
  * decoder_fifo_t
  *****************************************************************************
  * This rotative FIFO contains PES packets that are to be decoded.
  *****************************************************************************/
-typedef struct decoder_fifo_s
+struct decoder_fifo_t
 {
+    VLC_COMMON_MEMBERS
+
     /* Thread structures */
-    vlc_mutex_t             data_lock;                     /* fifo data lock */
-    vlc_cond_t              data_wait;     /* fifo data conditional variable */
+    vlc_mutex_t         data_lock;                         /* fifo data lock */
+    vlc_cond_t          data_wait;         /* fifo data conditional variable */
 
     /* Data */
-    pes_packet_t *          p_first;
-    pes_packet_t **         pp_last;
-    int                     i_depth;   /* number of PES packets in the stack */
+    pes_packet_t *      p_first;
+    pes_packet_t **     pp_last;
+    int                 i_depth;       /* number of PES packets in the stack */
 
     /* Communication interface between input and decoders */
-    boolean_t               b_die;          /* the decoder should return now */
-    boolean_t               b_error;      /* the decoder is in an error loop */
-    struct input_buffers_s *p_packets_mgt;   /* packets management services
-                                              * data */
-} decoder_fifo_t;
+    input_buffers_t    *p_packets_mgt;   /* packets management services data */
+
+    /* Standard pointers given to the decoders as a toolbox. */
+    uint16_t            i_id;
+    vlc_fourcc_t        i_fourcc;
+    es_sys_t *          p_demux_data;
+    stream_ctrl_t *     p_stream_ctrl;
+    sout_instance_t *   p_sout;
+    void *              p_waveformatex;
+    void *              p_bitmapinfoheader;
+
+    /* Module properties */
+    module_t *              p_module;
+    int                 ( * pf_run ) ( decoder_fifo_t * );
+};
 
 /*****************************************************************************
  * bit_fifo_t : bit fifo descriptor
@@ -124,9 +121,9 @@ typedef struct decoder_fifo_s
  * This type describes a bit fifo used to store bits while working with the
  * input stream at the bit level.
  *****************************************************************************/
-typedef u32         WORD_TYPE;
+typedef uint32_t    WORD_TYPE;
 
-typedef struct bit_fifo_s
+typedef struct bit_fifo_t
 {
     /* This unsigned integer allows us to work at the bit level. This buffer
      * can contain 32 bits, and the used space can be found on the MSb's side
@@ -144,36 +141,37 @@ typedef struct bit_fifo_s
  * This type, based on a PES stream, includes all the structures needed to
  * handle the input stream like a bit stream.
  *****************************************************************************/
-typedef struct bit_stream_s
+struct bit_stream_t
 {
     /*
      * Bit structures
      */
-    bit_fifo_t              fifo;
+    bit_fifo_t       fifo;
 
     /*
      * Input structures
      */
     /* The decoder fifo contains the data of the PES stream */
-    decoder_fifo_t *        p_decoder_fifo;
+    decoder_fifo_t * p_decoder_fifo;
 
     /* Callback to the decoder used when changing data packets ; set
      * to NULL if your decoder doesn't need it. */
-    void                 (* pf_bitstream_callback)( struct bit_stream_s *,
-                                                    boolean_t b_new_pes );
+    void          (* pf_bitstream_callback)( bit_stream_t *, vlc_bool_t );
     /* Optional argument to the callback */
-    void *                  p_callback_arg;
+    void *           p_callback_arg;
 
     /*
      * PTS retrieval
      */
-    mtime_t                 i_pts, i_dts;
-    byte_t *                p_pts_validity;
+    mtime_t          i_pts, i_dts;
+    byte_t *         p_pts_validity;
 
     /*
      * Byte structures
      */
-    /* Current data packet (in the current PES packet of the PES stream) */
+    /* Current PES packet (extracted from the PES stream) */
+    pes_packet_t *          p_pes;
+    /* Current data packet (in the current PES packet) */
     data_packet_t *         p_data;
     /* Pointer to the next byte that is to be read (in the current packet) */
     byte_t *                p_byte;
@@ -182,7 +180,7 @@ typedef struct bit_stream_s
     /* Temporary buffer in case we're not aligned when changing data packets */
     WORD_TYPE               i_showbits_buffer;
     data_packet_t           showbits_data;
-} bit_stream_t;
+};
 
 /*****************************************************************************
  * Inline functions used by the decoders to read bit_stream_t
@@ -200,12 +198,12 @@ typedef struct bit_stream_s
  * results will be returned. Use RealignBits() if unsure.
  */
 
-#if (WORD_TYPE == u32)
+#if (WORD_TYPE == uint32_t)
 #   define WORD_AT      U32_AT
-#   define WORD_SIGNED  s32
-#elif (WORD_TYPE == u64)
+#   define WORD_SIGNED  int32_t
+#elif (WORD_TYPE == uint64_t)
 #   define WORD_AT      U64_AT
-#   define WORD_SIGNED  s64
+#   define WORD_SIGNED  int64_t
 #else
 #   error Unsupported WORD_TYPE
 #endif
@@ -213,26 +211,17 @@ typedef struct bit_stream_s
 /*****************************************************************************
  * Prototypes from input_ext-dec.c
  *****************************************************************************/
-#ifndef __PLUGIN__
-void InitBitstream  ( struct bit_stream_s *, struct decoder_fifo_s *,
-                      void (* pf_bitstream_callback)( struct bit_stream_s *,
-                                                      boolean_t ),
-                      void * p_callback_arg );
-boolean_t NextDataPacket( struct decoder_fifo_s *, struct data_packet_s ** );
-void BitstreamNextDataPacket( struct bit_stream_s * );
-u32  UnalignedShowBits( struct bit_stream_s *, unsigned int );
-void UnalignedRemoveBits( struct bit_stream_s * );
-u32  UnalignedGetBits( struct bit_stream_s *, unsigned int );
-void CurrentPTS( struct bit_stream_s *, mtime_t *, mtime_t * );
-#else
-#   define InitBitstream p_symbols->InitBitstream
-#   define NextDataPacket p_symbols->NextDataPacket
-#   define BitstreamNextDataPacket p_symbols->BitstreamNextDataPacket
-#   define UnalignedShowBits p_symbols->UnalignedShowBits
-#   define UnalignedRemoveBits p_symbols->UnalignedRemoveBits
-#   define UnalignedGetBits p_symbols->UnalignedGetBits
-#   define CurrentPTS p_symbols->CurrentPTS
-#endif
+VLC_EXPORT( void, input_ExtractPES,        ( decoder_fifo_t *, pes_packet_t ** ) );
+VLC_EXPORT( void, input_DeletePES,         ( input_buffers_t *, pes_packet_t * ) );
+VLC_EXPORT( int, InitBitstream, ( bit_stream_t *, decoder_fifo_t *, void ( * )( bit_stream_t *, vlc_bool_t ), void * p_callback_arg ) );
+VLC_EXPORT( vlc_bool_t, NextDataPacket,    ( decoder_fifo_t *, bit_stream_t * ) );
+VLC_EXPORT( void, BitstreamNextDataPacket, ( bit_stream_t * ) );
+VLC_EXPORT( uint32_t, UnalignedShowBits,   ( bit_stream_t *, unsigned int ) );
+VLC_EXPORT( void, UnalignedRemoveBits,     ( bit_stream_t * ) );
+VLC_EXPORT( uint32_t, UnalignedGetBits,    ( bit_stream_t *, unsigned int ) );
+VLC_EXPORT( void, CloseBitstream,          ( bit_stream_t * ) );
+VLC_EXPORT( void, CurrentPTS,              ( bit_stream_t *, mtime_t *, mtime_t * ) );
+VLC_EXPORT( void, NextPTS,                 ( bit_stream_t *, mtime_t *, mtime_t * ) );
 
 /*****************************************************************************
  * AlignWord : fill in the bit buffer so that the byte pointer be aligned
@@ -265,10 +254,10 @@ static inline void AlignWord( bit_stream_t * p_bit_stream )
 /*****************************************************************************
  * ShowBits : return i_bits bits from the bit stream
  *****************************************************************************/
-static inline u32 ShowBits( bit_stream_t * p_bit_stream,
-                            unsigned int i_bits )
+static inline uint32_t ShowBits( bit_stream_t * p_bit_stream,
+                                 unsigned int i_bits )
 {
-    if( p_bit_stream->fifo.i_available >= i_bits )
+    if( (unsigned int)p_bit_stream->fifo.i_available >= i_bits )
     {
         return( p_bit_stream->fifo.buffer >> (8 * sizeof(WORD_TYPE) - i_bits) );
     }
@@ -288,10 +277,10 @@ static inline u32 ShowBits( bit_stream_t * p_bit_stream,
  * ShowSignedBits : return i_bits bits from the bit stream, using signed
  *                  arithmetic
  *****************************************************************************/
-static inline s32 ShowSignedBits( bit_stream_t * p_bit_stream,
-                                  unsigned int i_bits )
+static inline int32_t ShowSignedBits( bit_stream_t * p_bit_stream,
+                                      unsigned int i_bits )
 {
-    if( p_bit_stream->fifo.i_available >= i_bits )
+    if( (unsigned int)p_bit_stream->fifo.i_available >= i_bits )
     {
         return( (WORD_SIGNED)p_bit_stream->fifo.buffer
                     >> (8 * sizeof(WORD_TYPE) - i_bits) );
@@ -321,7 +310,8 @@ static inline void RemoveBits( bit_stream_t * p_bit_stream,
     {
         p_bit_stream->fifo.buffer = WORD_AT( p_bit_stream->p_byte )
                                         << ( -p_bit_stream->fifo.i_available );
-        ((WORD_TYPE *)p_bit_stream->p_byte)++;
+        p_bit_stream->p_byte =
+                   (byte_t *) ( ((WORD_TYPE *)p_bit_stream->p_byte) + 1 );
         p_bit_stream->fifo.i_available += sizeof(WORD_TYPE) * 8;
         return;
     }
@@ -333,7 +323,7 @@ static inline void RemoveBits( bit_stream_t * p_bit_stream,
  * RemoveBits32 : removes 32 bits from the bit buffer (and as a side effect,
  *                refill it)
  *****************************************************************************/
-#if (WORD_TYPE == u32)
+#if (WORD_TYPE == uint32_t)
 static inline void RemoveBits32( bit_stream_t * p_bit_stream )
 {
     if( p_bit_stream->p_byte <= p_bit_stream->p_end - sizeof(WORD_TYPE) )
@@ -342,11 +332,13 @@ static inline void RemoveBits32( bit_stream_t * p_bit_stream )
         {
             p_bit_stream->fifo.buffer = WORD_AT( p_bit_stream->p_byte )
                             << (32 - p_bit_stream->fifo.i_available);
-            ((WORD_TYPE *)p_bit_stream->p_byte)++;
+            p_bit_stream->p_byte =
+                       (byte_t *) ( ((WORD_TYPE *)p_bit_stream->p_byte) + 1 );
             return;
         }
 
-        ((WORD_TYPE *)p_bit_stream->p_byte)++;
+        p_bit_stream->p_byte =
+                   (byte_t *) ( ((WORD_TYPE *)p_bit_stream->p_byte) + 1 );
         return;
     }
 
@@ -361,10 +353,10 @@ static inline void RemoveBits32( bit_stream_t * p_bit_stream )
  * GetBits : returns i_bits bits from the bit stream and removes them
  *           XXX: do not use for 32 bits, see GetBits32
  *****************************************************************************/
-static inline u32 GetBits( bit_stream_t * p_bit_stream,
-                           unsigned int i_bits )
+static inline uint32_t GetBits( bit_stream_t * p_bit_stream,
+                                unsigned int i_bits )
 {
-    u32             i_result;
+    uint32_t        i_result;
 
     p_bit_stream->fifo.i_available -= i_bits;
 
@@ -381,7 +373,8 @@ static inline u32 GetBits( bit_stream_t * p_bit_stream,
         i_result = p_bit_stream->fifo.buffer
                         >> (8 * sizeof(WORD_TYPE) - i_bits);
         p_bit_stream->fifo.buffer = WORD_AT( p_bit_stream->p_byte );
-        ((WORD_TYPE *)p_bit_stream->p_byte)++;
+        p_bit_stream->p_byte =
+                   (byte_t *) ( ((WORD_TYPE *)p_bit_stream->p_byte) + 1 );
         i_result |= p_bit_stream->fifo.buffer
                         >> (8 * sizeof(WORD_TYPE)
                                      + p_bit_stream->fifo.i_available);
@@ -398,12 +391,12 @@ static inline u32 GetBits( bit_stream_t * p_bit_stream,
  *                 using signed arithmetic
  *                 XXX: do not use for 32 bits
  *****************************************************************************/
-static inline s32 GetSignedBits( bit_stream_t * p_bit_stream,
-                                 unsigned int i_bits )
+static inline int32_t GetSignedBits( bit_stream_t * p_bit_stream,
+                                     unsigned int i_bits )
 {
-    if( p_bit_stream->fifo.i_available >= i_bits )
+    if( (unsigned int)p_bit_stream->fifo.i_available >= i_bits )
     {
-        s32             i_result;
+        int32_t         i_result;
 
         p_bit_stream->fifo.i_available -= i_bits;
         i_result = (WORD_SIGNED)p_bit_stream->fifo.buffer
@@ -420,10 +413,10 @@ static inline s32 GetSignedBits( bit_stream_t * p_bit_stream,
 /*****************************************************************************
  * GetBits32 : returns 32 bits from the bit stream and removes them
  *****************************************************************************/
-#if (WORD_TYPE == u32)
-static inline u32 GetBits32( bit_stream_t * p_bit_stream )
+#if (WORD_TYPE == uint32_t)
+static inline uint32_t GetBits32( bit_stream_t * p_bit_stream )
 {
-    u32             i_result;
+    uint32_t        i_result;
 
     if( p_bit_stream->fifo.i_available == 32 )
     {
@@ -439,7 +432,8 @@ static inline u32 GetBits32( bit_stream_t * p_bit_stream )
         {
             i_result = p_bit_stream->fifo.buffer;
             p_bit_stream->fifo.buffer = WORD_AT( p_bit_stream->p_byte );
-            ((WORD_TYPE *)p_bit_stream->p_byte)++;
+            p_bit_stream->p_byte =
+                       (byte_t *) ( ((WORD_TYPE *)p_bit_stream->p_byte) + 1 );
             i_result |= p_bit_stream->fifo.buffer
                              >> (p_bit_stream->fifo.i_available);
             p_bit_stream->fifo.buffer <<= (32 - p_bit_stream->fifo.i_available);
@@ -447,7 +441,8 @@ static inline u32 GetBits32( bit_stream_t * p_bit_stream )
         }
 
         i_result = WORD_AT( p_bit_stream->p_byte );
-        ((WORD_TYPE *)p_bit_stream->p_byte)++;
+        p_bit_stream->p_byte =
+                   (byte_t *) ( ((WORD_TYPE *)p_bit_stream->p_byte) + 1 );
         return( i_result );
     }
 
@@ -490,17 +485,19 @@ static inline void GetChunk( bit_stream_t * p_bit_stream,
         p_bit_stream->fifo.i_available -= 8;
     }
 
-    if( (i_available = p_bit_stream->p_end - p_bit_stream->p_byte)
-            >= i_buf_len )
+    i_available = p_bit_stream->p_end - p_bit_stream->p_byte;
+    if( i_available >= (ptrdiff_t)i_buf_len )
     {
-        FAST_MEMCPY( p_buffer, p_bit_stream->p_byte, i_buf_len );
+        p_bit_stream->p_decoder_fifo->p_vlc->pf_memcpy( p_buffer,
+                                           p_bit_stream->p_byte, i_buf_len );
         p_bit_stream->p_byte += i_buf_len;
     }
     else
     {
         do
         {
-            FAST_MEMCPY( p_buffer, p_bit_stream->p_byte, i_available );
+            p_bit_stream->p_decoder_fifo->p_vlc->pf_memcpy( p_buffer,
+                                           p_bit_stream->p_byte, i_available );
             p_bit_stream->p_byte = p_bit_stream->p_end;
             p_buffer += i_available;
             i_buf_len -= i_available;
@@ -509,11 +506,12 @@ static inline void GetChunk( bit_stream_t * p_bit_stream,
                 return;
         }
         while( (i_available = p_bit_stream->p_end - p_bit_stream->p_byte)
-                <= i_buf_len );
+                <= (ptrdiff_t)i_buf_len );
 
         if( i_buf_len )
         {
-            FAST_MEMCPY( p_buffer, p_bit_stream->p_byte, i_buf_len );
+            p_bit_stream->p_decoder_fifo->p_vlc->pf_memcpy( p_buffer,
+                                           p_bit_stream->p_byte, i_buf_len );
             p_bit_stream->p_byte += i_buf_len;
         }
     }
@@ -529,27 +527,9 @@ static inline void GetChunk( bit_stream_t * p_bit_stream,
  * Communication interface between input and decoders
  */
 
-/*****************************************************************************
- * decoder_config_t
- *****************************************************************************
- * Standard pointers given to the decoders as a toolbox.
- *****************************************************************************/
-typedef struct decoder_config_s
-{
-    u16                     i_id;
-    u8                      i_type;         /* type of the elementary stream */
-
-    void *                  p_demux_data;
-    struct stream_ctrl_s *  p_stream_ctrl;
-    struct decoder_fifo_s * p_decoder_fifo;
-} decoder_config_t;
-
 /*****************************************************************************
  * Prototypes from input_dec.c
  *****************************************************************************/
-#ifndef __PLUGIN__
-void DecoderError      ( struct decoder_fifo_s * p_fifo );
-#else
-#   define DecoderError p_symbols->DecoderError
-#endif
+VLC_EXPORT( void, DecoderError, ( decoder_fifo_t * p_fifo ) );
 
+#endif /* "input_ext-dec.h" */
