@@ -9,7 +9,7 @@
  * Functions prototyped are implemented in interface/mtime.c.
  *****************************************************************************
  * Copyright (C) 1996, 1997, 1998, 1999, 2000 VideoLAN
- * $Id: mtime.h,v 1.11 2002/04/24 00:36:24 sam Exp $
+ * $Id: mtime.h,v 1.13 2002/11/11 14:39:11 sam Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *
@@ -36,7 +36,7 @@
  * returning the date of the first image to be displayed. It can be used in
  * comparaison with other values: all existing dates will be earlier.
  *****************************************************************************/
-#define LAST_MDATE ((mtime_t)((u64)(-1)/2))
+#define LAST_MDATE ((mtime_t)((uint64_t)(-1)/2))
 
 /*****************************************************************************
  * MSTRTIME_MAX_SIZE: maximum possible size of mstrtime
@@ -50,14 +50,8 @@
 /*****************************************************************************
  * Prototypes
  *****************************************************************************/
-#ifndef __PLUGIN__
-char *  mstrtime ( char *psz_buffer, mtime_t date );
-mtime_t mdate    ( void );
-void    mwait    ( mtime_t date );
-void    msleep   ( mtime_t delay );
-#else
-#   define msleep    p_symbols->msleep
-#   define mdate     p_symbols->mdate
-#   define mstrtime  p_symbols->mstrtime
-#endif
+VLC_EXPORT( char *,  mstrtime, ( char *psz_buffer, mtime_t date ) );
+VLC_EXPORT( mtime_t, mdate,    ( void ) );
+VLC_EXPORT( void,    mwait,    ( mtime_t date ) );
+VLC_EXPORT( void,    msleep,   ( mtime_t delay ) );
 
