@@ -206,7 +206,6 @@ int E_(Open) ( vlc_object_t *p_this )
         return -1;
     }
 
-
     /* Get antenna configuration options */
     b_diseqc = config_GetInt( p_input, "diseqc" );
     i_lnb_lof1 = config_GetInt( p_input, "lnb-lof1" );
@@ -437,7 +436,7 @@ int SatelliteSetProgram( input_thread_t    * p_input,
 
     /* Update the navigation variables without triggering a callback */
     val.i_int = p_new_prg->i_number;
-    var_Change( p_input, "program", VLC_VAR_SETVALUE, &val );
+    var_Change( p_input, "program", VLC_VAR_SETVALUE, &val, NULL );
 
     return 0;
 }
