@@ -2,7 +2,7 @@
  * rc.c : remote control stdin/stdout plugin for vlc
  *****************************************************************************
  * Copyright (C) 2001 VideoLAN
- * $Id: rc.c,v 1.34 2003/05/15 22:27:37 massiot Exp $
+ * $Id: rc.c,v 1.37 2003/07/28 07:16:50 fenrir Exp $
  *
  * Authors: Peter Surda <shurdeek@panorama.sth.ac.at>
  *
@@ -46,7 +46,7 @@
 #endif
 #include <sys/types.h>
 
-#include "error.h"
+#include "vlc_error.h"
 
 #define MAX_LINE_LENGTH 256
 
@@ -527,8 +527,19 @@ static int Input( vlc_object_t *p_this, char const *psz_cmd,
     }
     else if( !strcmp( psz_cmd, "seek" ) )
     {
-        input_Seek( p_input, atoi( newval.psz_string ),
-                    INPUT_SEEK_SECONDS | INPUT_SEEK_SET );
+        if( strlen( newval.psz_string ) > 0 &&
+            newval.psz_string[strlen( newval.psz_string ) - 1] == '%' )
+        {
+            input_Seek( p_input, atoi( newval.psz_string ),
+                        INPUT_SEEK_PERCENT | INPUT_SEEK_SET );
+        }
+        else
+        {
+            input_Seek( p_input, atoi( newval.psz_string ),
+                        INPUT_SEEK_SECONDS | INPUT_SEEK_SET );
+        }
+        vlc_object_release( p_input );
+        return VLC_SUCCESS;
     }
     else if( !strcmp( psz_cmd, "chapter" ) ||
              !strcmp( psz_cmd, "chapter_n" ) ||
@@ -675,7 +686,7 @@ static int Playlist( vlc_object_t *p_this, char const *psz_cmd,
     else if( !strcmp( psz_cmd, "add" ) )
     {
         printf( "trying to add %s to playlist\n", newval.psz_string );
-        playlist_Add( p_playlist, newval.psz_string,
+        playlist_Add( p_playlist, newval.psz_string, NULL, 0,
                       PLAYLIST_GO|PLAYLIST_APPEND, PLAYLIST_END );
     }
     else if( !strcmp( psz_cmd, "playlist" ) )
