@@ -2,7 +2,7 @@
  * system.h: MPEG demultiplexing.
  *****************************************************************************
  * Copyright (C) 1999-2002 VideoLAN
- * $Id: system.h,v 1.6 2003/02/23 18:07:30 fenrir Exp $
+ * $Id: system.h,v 1.8 2003/07/13 12:35:13 massiot Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -46,11 +46,14 @@
 #define MPEG2_VIDEO_ES      0x02
 #define MPEG1_AUDIO_ES      0x03
 #define MPEG2_AUDIO_ES      0x04
-#define A52DVB_AUDIO_ES     0x06
+/* This one is "private PES". It may carry teletext and DVB, and we ought
+ * to check for the presence of a descriptor, but we don't. */
+#define PESDVB_AUDIO_ES     0x06
 
 #define MPEG4_VIDEO_ES      0x10
 #define MPEG4_AUDIO_ES      0x11
 
+#define MPEG2_MOTO_VIDEO_ES 0x80
 #define A52_AUDIO_ES        0x81
 /* These ones might violate the usage : */
 #define DVD_SPU_ES          0x82
