@@ -2,7 +2,7 @@
  * VlcWrapper.h: BeOS plugin for vlc (derived from MacOS X port)
  *****************************************************************************
  * Copyright (C) 2001 VideoLAN
- * $Id: VlcWrapper.h,v 1.20 2003/02/09 11:51:36 titer Exp $
+ * $Id: VlcWrapper.h,v 1.26 2003/06/22 00:40:18 titer Exp $
  *
  * Authors: Florian G. Pflug <fgp@phlo.org>
  *          Jon Lech Johansen <jon-vl@nanocrew.net>
@@ -25,6 +25,8 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  *****************************************************************************/
 
+#include <SupportKit.h>
+
 #define SEEKSLIDER_RANGE 2048
 
 class InterfaceWindow;
@@ -42,13 +44,14 @@ struct intf_sys_t
     vlc_bool_t           b_loop;
     vlc_bool_t           b_mute;
     int                  i_part;
-    audio_volume_t       i_saved_volume;
     int                  i_channel;
-    bool                 b_dvdmenus;
     
     VlcWrapper *         p_wrapper;
 };
 
+/* Necessary because of i18n */
+const char * _AddEllipsis( char * string );
+
 /*****************************************************************************
  * VlcWrapper
  *****************************************************************************
@@ -62,14 +65,12 @@ public:
     VlcWrapper( intf_thread_t *p_intf );
     ~VlcWrapper();
     
-    bool UpdateInput();
-    
     /* Input */
+    void         UpdateInput();
     bool         HasInput();
     int          InputStatus();
     int          InputRate();
-    void         InputSlower();
-    void         InputFaster();
+    void         InputSetRate( int rate );
     BList *      GetChannels( int i_cat );
     void         ToggleLanguage( int i_language );
     void         ToggleSubtitle( int i_subtitle );
@@ -123,6 +124,7 @@ public:
     bool           IsMuted();
 
     /* DVD */
+    bool    IsUsingMenus();
     bool    HasTitles();
     BList * GetTitles();
     void    PrevTitle();
