@@ -2,7 +2,7 @@
  * httpd.c
  *****************************************************************************
  * Copyright (C) 2001-2003 VideoLAN
- * $Id: httpd.c,v 1.7 2003/03/06 11:09:56 fenrir Exp $
+ * $Id: httpd.c,v 1.25 2003/08/11 20:18:02 fenrir Exp $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -23,17 +23,18 @@
 
 /*****************************************************************************
  * Preamble
+ *
+ * TODO:
+ *  - make that two distinct host:port use different daemon
  *****************************************************************************/
 #include <stdlib.h>
+#include <vlc/vlc.h>
 
-#include <sys/types.h>
 #include <sys/stat.h>
 
-#include <string.h>
 #include <errno.h>
 #include <fcntl.h>
 
-#include <vlc/vlc.h>
 #include "httpd.h"
 
 #ifdef HAVE_SYS_TIME_H
@@ -74,6 +75,7 @@
 #define HTTPD_MAX_CONNECTION    512
 #define HTTPD_CONNECTION_MAX_UNUSED 10000000
 
+
 #define FREE( p ) if( p ) { free( p); (p) = NULL; }
 
 #if defined( WIN32 ) || defined( UNDER_CE )
@@ -95,6 +97,7 @@ vlc_module_begin();
     set_description( _("HTTP 1.0 daemon") );
     set_capability( "httpd", 42 );
     set_callbacks( Open, Close );
+    var_Create( p_module->p_libvlc, "httpd", VLC_VAR_MUTEX );
 vlc_module_end();
 
 /*****************************************************************************
@@ -106,7 +109,8 @@ static void             UnregisterHost  ( httpd_t *, httpd_host_t * );
 static httpd_file_t     *RegisterFile   ( httpd_t *,
                                           char *psz_file, char *psz_mime,
                                           char *psz_user, char *psz_password,
-                                          httpd_file_callback pf_fill,
+                                          httpd_file_callback pf_get,
+                                          httpd_file_callback pf_post,
                                           httpd_file_callback_args_t *p_args );
 static void             UnregisterFile  ( httpd_t *, httpd_file_t * );
 
@@ -117,7 +121,7 @@ static httpd_stream_t   *RegisterStream ( httpd_t *,
 static int              SendStream      ( httpd_t *, httpd_stream_t *, uint8_t *, int );
 static int              HeaderStream    ( httpd_t *, httpd_stream_t *, uint8_t *, int );
 static void             UnregisterStream( httpd_t *, httpd_stream_t* );
-
+static int              Control         ( httpd_t *, int , void*, void* );
 /*****************************************************************************
  * Internal definitions
  *****************************************************************************/
@@ -133,8 +137,11 @@ struct httpd_host_t
 
 };
 
-#define HTTPD_AUTHENTICATE_NONE     0
-#define HTTPD_AUTHENTICATE_BASIC    1
+enum httpd_authenticate_e
+{
+    HTTPD_AUTHENTICATE_NONE = 0,
+    HTTPD_AUTHENTICATE_BASIC = 1
+};
 
 //typedef httpd_file_t httpd_stream_t;
 
@@ -153,7 +160,8 @@ struct httpd_file_t
     vlc_bool_t  b_stream;               /* if false: httpd will retreive data by a callback
                                               true:  it's up to the program to give data to httpd */
     void                *p_sys;         /* provided for user */
-    httpd_file_callback pf_fill;       /* it should allocate and fill *pp_data and *pi_data */
+    httpd_file_callback pf_get;         /* it should allocate and fill *pp_data and *pi_data */
+    httpd_file_callback pf_post;        /* it should allocate and fill *pp_data and *pi_data */
 
     /* private */
 
@@ -169,10 +177,21 @@ struct httpd_file_t
 };
 
 
-#define HTTPD_CONNECTION_RECEIVING_REQUEST      1
-#define HTTPD_CONNECTION_SENDING_HEADER         2
-#define HTTPD_CONNECTION_SENDING_FILE           3
-#define HTTPD_CONNECTION_SENDING_STREAM         4
+enum httpd_connection_state_e
+{
+    HTTPD_CONNECTION_RECEIVING_REQUEST = 1,
+    HTTPD_CONNECTION_SENDING_HEADER = 2,
+    HTTPD_CONNECTION_SENDING_FILE = 3,
+    HTTPD_CONNECTION_SENDING_STREAM = 4,
+    HTTPD_CONNECTION_TO_BE_CLOSED = 5
+};
+
+enum httpd_connection_method_e
+{
+    HTTPD_CONNECTION_METHOD_GET = 1,
+    HTTPD_CONNECTION_METHOD_POST = 2,
+    HTTPD_CONNECTION_METHOD_HEAD =3
+};
 
 typedef struct httpd_connection_s
 {
@@ -184,12 +203,16 @@ typedef struct httpd_connection_s
     mtime_t i_last_activity_date;
 
     int    i_state;
+    int    i_method;       /* get/post */
 
     char    *psz_file;      // file to be send
     int     i_http_error;   // error to be send with the file
     char    *psz_user;      // if Authorization in the request header
     char    *psz_password;
 
+    uint8_t *p_request;     // whith get: ?<*>, with post: main data
+    int      i_request_size;
+
     httpd_file_t    *p_file;
 
     /* used while sending header and file */
@@ -201,6 +224,15 @@ typedef struct httpd_connection_s
     int64_t i_stream_pos;   /* absolute pos in stream */
 } httpd_connection_t;
 
+/* Linked List of banned IP */
+typedef struct httpd_banned_ip_s
+{
+    struct httpd_banned_ip_s *p_next;
+    struct httpd_banned_ip_s *p_prev;
+
+    char *psz_ip;
+
+} httpd_banned_ip_t;
 /*
  * The httpd thread
  */
@@ -219,11 +251,20 @@ struct httpd_sys_t
     vlc_mutex_t             connection_lock;
     int                     i_connection_count;
     httpd_connection_t      *p_first_connection;
+
+    vlc_mutex_t             ban_lock;
+    int                     i_banned_ip_count;
+    httpd_banned_ip_t       *p_first_banned_ip;
 };
 
 static void httpd_Thread( httpd_sys_t *p_httpt );
 static void httpd_ConnnectionNew( httpd_sys_t *, int , struct sockaddr_in * );
 static void httpd_ConnnectionClose( httpd_sys_t *, httpd_connection_t * );
+static int httpd_UnbanIP( httpd_sys_t *, httpd_banned_ip_t *);
+#if 0
+static int httpd_BanIP( httpd_sys_t *, char *);
+#endif
+static httpd_banned_ip_t *httpd_GetbannedIP( httpd_sys_t *, char * );
 
 /*****************************************************************************
  * Open:
@@ -257,6 +298,10 @@ static int Open( vlc_object_t *p_this )
     p_httpt->i_connection_count = 0;
     p_httpt->p_first_connection = NULL;
 
+    vlc_mutex_init( p_httpd, &p_httpt->ban_lock );
+    p_httpt->i_banned_ip_count = 0;
+    p_httpt->p_first_banned_ip = NULL;
+
     /* start the thread */
     if( vlc_thread_create( p_httpt, "httpd thread",
                            httpd_Thread, VLC_THREAD_PRIORITY_LOW, VLC_FALSE ) )
@@ -266,6 +311,7 @@ static int Open( vlc_object_t *p_this )
         vlc_mutex_destroy( &p_httpt->host_lock );
         vlc_mutex_destroy( &p_httpt->file_lock );
         vlc_mutex_destroy( &p_httpt->connection_lock );
+        vlc_mutex_destroy( &p_httpt->ban_lock );
 
         vlc_object_destroy( p_httpt );
         return( VLC_EGENERIC );
@@ -282,6 +328,7 @@ static int Open( vlc_object_t *p_this )
     p_httpd->pf_header_stream   = HeaderStream;
     p_httpd->pf_send_stream     = SendStream;
     p_httpd->pf_unregister_stream=UnregisterStream;
+    p_httpd->pf_control         = Control;
 
     return( VLC_SUCCESS );
 }
@@ -295,6 +342,8 @@ static void Close( vlc_object_t * p_this )
     httpd_sys_t *p_httpt = p_httpd->p_sys;
 
     httpd_connection_t *p_con;
+    httpd_banned_ip_t *p_banned_ip;
+
     int i;
 
     p_httpt->b_die = 1;
@@ -351,6 +400,13 @@ static void Close( vlc_object_t * p_this )
         httpd_ConnnectionClose( p_httpt, p_con );
     }
 
+    /* Free all banned IP */
+    vlc_mutex_destroy( &p_httpt->ban_lock );
+    while( ( p_banned_ip = p_httpt->p_first_banned_ip))
+    {
+        httpd_UnbanIP(p_httpt,p_banned_ip);
+    }
+
     msg_Info( p_httpd, "httpd instance closed" );
     vlc_object_destroy( p_httpt );
 }
@@ -427,7 +483,8 @@ static httpd_host_t *_RegisterHost( httpd_sys_t *p_httpt, char *psz_host_addr, i
     for( i = 0; i < p_httpt->i_host_count; i++ )
     {
         if( p_httpt->host[i]->sock.sin_port == sock.sin_port &&
-            p_httpt->host[i]->sock.sin_addr.s_addr == sock.sin_addr.s_addr )
+            ( p_httpt->host[i]->sock.sin_addr.s_addr == INADDR_ANY ||
+            p_httpt->host[i]->sock.sin_addr.s_addr == sock.sin_addr.s_addr ) )
         {
             break;
         }
@@ -611,7 +668,8 @@ static void __RegisterFile( httpd_sys_t *p_httpt, httpd_file_t *p_file )
 static httpd_file_t    *_RegisterFile( httpd_sys_t *p_httpt,
                                        char *psz_file, char *psz_mime,
                                        char *psz_user, char *psz_password,
-                                       httpd_file_callback pf_fill,
+                                       httpd_file_callback pf_get,
+                                       httpd_file_callback pf_post,
                                        httpd_file_callback_args_t *p_args )
 {
     httpd_file_t    *p_file;
@@ -651,7 +709,8 @@ static httpd_file_t    *_RegisterFile( httpd_sys_t *p_httpt,
 
     p_file->b_stream          = VLC_FALSE;
     p_file->p_sys             = p_args;
-    p_file->pf_fill           = pf_fill;
+    p_file->pf_get            = pf_get;
+    p_file->pf_post           = pf_post;
 
     p_file->i_buffer_size     = 0;
     p_file->i_buffer_last_pos = 0;
@@ -670,12 +729,13 @@ static httpd_file_t    *_RegisterFile( httpd_sys_t *p_httpt,
 static httpd_file_t     *RegisterFile( httpd_t *p_httpd,
                                        char *psz_file, char *psz_mime,
                                        char *psz_user, char *psz_password,
-                                       httpd_file_callback pf_fill,
+                                       httpd_file_callback pf_get,
+                                       httpd_file_callback pf_post,
                                        httpd_file_callback_args_t *p_args )
 {
     return( _RegisterFile( p_httpd->p_sys,
                            psz_file, psz_mime, psz_user, psz_password,
-                           pf_fill, p_args ) );
+                           pf_get, pf_post, p_args ) );
 }
 
 static httpd_stream_t  *_RegisterStream( httpd_sys_t *p_httpt,
@@ -719,9 +779,10 @@ static httpd_stream_t  *_RegisterStream( httpd_sys_t *p_httpt,
 
     p_stream->b_stream        = VLC_TRUE;
     p_stream->p_sys           = NULL;
-    p_stream->pf_fill         = NULL;
+    p_stream->pf_get          = NULL;
+    p_stream->pf_post         = NULL;
 
-    p_stream->i_buffer_size   = 1024*1024*10;
+    p_stream->i_buffer_size   = 5*1024*1024;
     p_stream->i_buffer_pos      = 0;
     p_stream->i_buffer_last_pos = 0;
     p_stream->p_buffer        = malloc( p_stream->i_buffer_size );
@@ -850,7 +911,7 @@ static int             _SendStream( httpd_sys_t *p_httpt, httpd_stream_t *p_stre
     {
         int i_copy;
 
-        i_copy = __MIN( i_data, p_stream->i_buffer_size - i_pos );
+        i_copy = __MIN( i_count, p_stream->i_buffer_size - i_pos );
 
         memcpy( &p_stream->p_buffer[i_pos],
                 p_data,
@@ -895,13 +956,145 @@ static int             HeaderStream( httpd_t *p_httpd, httpd_stream_t *p_stream,
     return( VLC_SUCCESS );
 }
 
+static void httpd_info_add_ss( httpd_info_t *p_info, char *name, char *value )
+{
+    if( p_info->i_count == 0 )
+    {
+        p_info->info = malloc( sizeof( httpd_val_t ) );
+    }
+    else
+    {
+        p_info->info =
+            realloc( p_info->info,
+                     sizeof( httpd_val_t ) * ( p_info->i_count + 1 ) );
+    }
+    p_info->info[p_info->i_count].psz_name  = strdup( name );
+    p_info->info[p_info->i_count].psz_value = strdup( value ? value : "(null)");
+    p_info->i_count++;
+}
+
+static void httpd_info_add_si( httpd_info_t *p_info, char *name, int i_value )
+{
+    char v[40];
+
+    sprintf( v, "%d", i_value );
+    httpd_info_add_ss( p_info, name, v );
+}
+
+static void httpd_info_add_sp( httpd_info_t *p_info, char *name, void *value )
+{
+    char v[40];
+
+    sprintf( v, "%p", value );
+    httpd_info_add_ss( p_info, name, v );
+}
+
+
+static int Control( httpd_t *p_httpd,
+                    int i_query, void *arg1, void *arg2 )
+{
+    httpd_sys_t  *p_httpt = p_httpd->p_sys;
+    httpd_info_t *p_info;
+    httpd_connection_t *p_con;
+    int i;
+    void *id;
+
+    switch( i_query )
+    {
+        case HTTPD_GET_HOSTS:
+            p_info = arg1;
+            p_info->i_count = 0;
+            vlc_mutex_lock( &p_httpt->host_lock );
+            for( i = 0; i < p_httpt->i_host_count; i++ )
+            {
+                httpd_info_add_sp( p_info,
+                                   "id", p_httpt->host[i] );
+                httpd_info_add_ss( p_info,
+                                   "host", p_httpt->host[i]->psz_host_addr );
+                httpd_info_add_ss( p_info,
+                                   "ip",
+                                   inet_ntoa(p_httpt->host[i]->sock.sin_addr));
+                httpd_info_add_si( p_info,
+                                   "port", p_httpt->host[i]->i_port );
+            }
+            vlc_mutex_unlock( &p_httpt->host_lock );
+            return VLC_SUCCESS;
+        case HTTPD_GET_URLS:
+            p_info = arg1;
+            p_info->i_count = 0;
+            /* we can't take file_lock */
+            for( i = 0; i < p_httpt->i_file_count; i++ )
+            {
+                httpd_info_add_sp( p_info,
+                                   "id", p_httpt->file[i] );
+                httpd_info_add_si( p_info,
+                                   "stream", p_httpt->file[i]->b_stream ? 1 : 0 );
+                httpd_info_add_ss( p_info,
+                                   "url", p_httpt->file[i]->psz_file );
+                httpd_info_add_ss( p_info,
+                                   "mime", p_httpt->file[i]->psz_mime );
+                httpd_info_add_si( p_info,
+                                   "protected", p_httpt->file[i]->psz_user ? 1 : 0 );
+                httpd_info_add_si( p_info,
+                                   "used", p_httpt->file[i]->i_ref );
+            }
+            return VLC_SUCCESS;
+        case HTTPD_GET_CONNECTIONS:
+            p_info = arg1;
+            p_info->i_count = 0;
+            /* we can't take lock */
+            for( p_con = p_httpt->p_first_connection;p_con != NULL; p_con = p_con->p_next )
+            {
+                if( p_con->i_state != HTTPD_CONNECTION_TO_BE_CLOSED )
+                {
+                    httpd_info_add_sp( p_info,
+                                       "id", p_con );
+                    httpd_info_add_ss( p_info,
+                                       "ip", inet_ntoa( p_con->sock.sin_addr ) );
+                    httpd_info_add_ss( p_info,
+                                       "url", p_con->psz_file );
+                    httpd_info_add_si( p_info,
+                                       "status", p_con->i_http_error );
+                }
+            }
+            return VLC_SUCCESS;
+        case HTTPD_GET_ACL:
+            p_info = arg1;
+            p_info->i_count = 0;
+            return VLC_EGENERIC;
+
+        case HTTPD_SET_CLOSE:
+            sscanf( arg1, "%p", &id );
+            fprintf( stderr, "Control: HTTPD_SET_CLOSE: id=%p", id );
+
+            for( p_con = p_httpt->p_first_connection;p_con != NULL; p_con = p_con->p_next )
+            {
+                if( (void*)p_con == id )
+                {
+                    /* XXX don't free p_con as it could be the one that it is sending ... */
+                    p_con->i_state = HTTPD_CONNECTION_TO_BE_CLOSED;
+                    return VLC_SUCCESS;
+                }
+            }
+            return VLC_EGENERIC;
+
+        case HTTPD_SET_ACL:
+            sscanf( arg1, "%p", &id );
+            fprintf( stderr, "Control: %p", id );
+        default:
+            return VLC_EGENERIC;
+    }
+}
+
 /****************************************************************************/
 /****************************************************************************/
 /****************************************************************************/
 /****************************************************************************/
 /****************************************************************************/
 
-static int  httpd_page_401_fill( httpd_file_callback_args_t *p_args, uint8_t **pp_data, int *pi_data )
+static int  httpd_page_401_get( httpd_file_callback_args_t *p_args,
+                                uint8_t *p_request, int i_request,
+                                uint8_t **pp_data, int *pi_data )
 {
     char *p;
 
@@ -922,7 +1115,9 @@ static int  httpd_page_401_fill( httpd_file_callback_args_t *p_args, uint8_t **p
 
     return VLC_SUCCESS;
 }
-static int  httpd_page_404_fill( httpd_file_callback_args_t *p_args, uint8_t **pp_data, int *pi_data )
+static int  httpd_page_404_get( httpd_file_callback_args_t *p_args,
+                                uint8_t *p_request, int i_request,
+                                uint8_t **pp_data, int *pi_data )
 {
     char *p;
 
@@ -944,116 +1139,100 @@ static int  httpd_page_404_fill( httpd_file_callback_args_t *p_args, uint8_t **p
     return VLC_SUCCESS;
 }
 
-static int  httpd_page_admin_fill( httpd_file_callback_args_t *p_args, uint8_t **pp_data, int *pi_data )
+#if 0
+static int httpd_BanIP( httpd_sys_t *p_httpt, char * psz_new_banned_ip)
 {
-    httpd_sys_t *p_httpt = (httpd_sys_t*)p_args;
-    httpd_connection_t *p_con;
-    char *p;
-    int i;
+    httpd_banned_ip_t *p_new_banned_ip ;
 
-    /* FIXME FIXME do not use static size FIXME FIXME*/
-    p = *pp_data = malloc( 8096 );
-
-    p += sprintf( p, "<html>\n" );
-    p += sprintf( p, "<head>\n" );
-    p += sprintf( p, "<title>VideoLAN Client Stream Output</title>\n" );
-    p += sprintf( p, "</head>\n" );
-    p += sprintf( p, "<body>\n" );
-    p += sprintf( p, "<h1><center>VideoLAN Client Stream Output</center></h1>\n" );
-    p += sprintf( p, "<h2><center>Admin page</center></h2>\n" );
-
-    /* general */
-    p += sprintf( p, "<h3>General state</h3>\n" );
-    p += sprintf( p, "<ul>\n" );
-    p += sprintf( p, "<li>Connection count: %d</li>\n", p_httpt->i_connection_count );
-    //p += sprintf( p, "<li>Total bandwith: %d</li>\n", -1 );
-    /*p += sprintf( p, "<li></li>\n" );*/
-    p += sprintf( p, "</ul>\n" );
-    /* host list */
-    vlc_mutex_lock( &p_httpt->host_lock );
-    p += sprintf( p, "<h3>Host list</h3>\n" );
-    p += sprintf( p, "<table border=\"1\" cellspacing=\"0\" >\n" );
-    p += sprintf( p, "<tr>\n<th>Host</th><th>Port</th><th>IP</th>\n</tr>\n" );
-
-    for( i = 0; i < p_httpt->i_host_count; i++ )
+    p_new_banned_ip = malloc( sizeof( httpd_banned_ip_t ) );
+    if( !p_new_banned_ip )
     {
-        p += sprintf( p, "<tr>\n" );
-        p += sprintf( p, "<td>%s</td>\n", p_httpt->host[i]->psz_host_addr );
-        p += sprintf( p, "<td>%d</td>\n", p_httpt->host[i]->i_port );
-        p += sprintf( p, "<td>%s</td>\n", inet_ntoa( p_httpt->host[i]->sock.sin_addr ) );
-        p += sprintf( p, "</tr>\n" );
+        return -1;
     }
-    p += sprintf( p, "</table>\n" );
-    vlc_mutex_unlock( &p_httpt->host_lock );
-
-    /* file list */
-    /* XXX do not take lock on file_lock */
-    p += sprintf( p, "<h3>File list</h3>\n" );
-    p += sprintf( p, "<table border=\"1\" cellspacing=\"0\" >\n" );
-    p += sprintf( p, "<tr>\n<th>Name</th><th>Mime</th><th>Protected</th><th>Used</th>\n</tr>\n" );
-
-    for( i = 0; i < p_httpt->i_file_count; i++ )
+    p_new_banned_ip->p_next=NULL;
+    p_new_banned_ip->psz_ip = malloc( strlen( psz_new_banned_ip ) + 1 );
+    if( !p_new_banned_ip->psz_ip )
     {
-        if( !p_httpt->file[i]->b_stream )
+        return -2;
+    }
+
+    strcpy( p_new_banned_ip->psz_ip, psz_new_banned_ip );
+
+    msg_Dbg( p_httpt, "Banning IP %s", psz_new_banned_ip );
+
+    if( p_httpt->p_first_banned_ip )
+    {
+        httpd_banned_ip_t *p_last;
+
+        p_last = p_httpt->p_first_banned_ip;
+        while( p_last->p_next )
         {
-            p += sprintf( p, "<tr>\n" );
-            p += sprintf( p, "<td>%s</td>\n", p_httpt->file[i]->psz_file );
-            p += sprintf( p, "<td>%s</td>\n", p_httpt->file[i]->psz_mime );
-            p += sprintf( p, "<td>%s</td>\n", p_httpt->file[i]->psz_user ? "Yes" : "No" );
-            p += sprintf( p, "<td>%d</td>\n", p_httpt->file[i]->i_ref);
-            p += sprintf( p, "</tr>\n" );
+            p_last = p_last->p_next;
         }
+
+        p_last->p_next = p_new_banned_ip;
+        p_new_banned_ip->p_prev = p_last;
     }
-    p += sprintf( p, "</table>\n" );
-
-    /* stream list */
-    /* XXX do not take lock on file_lock */
-    p += sprintf( p, "<h3>Stream list</h3>\n" );
-    p += sprintf( p, "<table border=\"1\" cellspacing=\"0\" >\n" );
-    p += sprintf( p, "<tr>\n<th>Name</th><th>Mime</th><th>Protected</th><th>Used</th>\n</tr>\n" );
-
-    for( i = 0; i < p_httpt->i_file_count; i++ )
+    else
     {
-        if( p_httpt->file[i]->b_stream )
+        p_new_banned_ip->p_prev = NULL;
+
+        p_httpt->p_first_banned_ip = p_new_banned_ip;
+    }
+
+    p_httpt->i_banned_ip_count++;
+    return 0;
+}
+#endif
+static httpd_banned_ip_t *httpd_GetbannedIP( httpd_sys_t *p_httpt, char *psz_ip )
+{
+    httpd_banned_ip_t *p_ip;
+
+    p_ip = p_httpt->p_first_banned_ip;
+
+    while( p_ip)
+    {
+        if( strcmp( psz_ip, p_ip->psz_ip ) == 0 )
         {
-            p += sprintf( p, "<tr>\n" );
-            p += sprintf( p, "<td>%s</td>\n", p_httpt->file[i]->psz_file );
-            p += sprintf( p, "<td>%s</td>\n", p_httpt->file[i]->psz_mime );
-            p += sprintf( p, "<td>%s</td>\n", p_httpt->file[i]->psz_user ? "Yes" : "No" );
-            p += sprintf( p, "<td>%d</td>\n", p_httpt->file[i]->i_ref);
-            p += sprintf( p, "</tr>\n" );
+            return p_ip;
         }
+        p_ip = p_ip->p_next;
     }
-    p += sprintf( p, "</table>\n" );
 
-    /* connection list */
-    /* XXX do not take lock on connection_lock */
-    p += sprintf( p, "<h3>Connection list</h3>\n" );
-    p += sprintf( p, "<table border=\"1\" cellspacing=\"0\" >\n" );
-    p += sprintf( p, "<tr>\n<th>IP</th><th>Requested File</th><th>Status</th>\n</tr>\n" );
-
-    for( p_con = p_httpt->p_first_connection;p_con != NULL; p_con = p_con->p_next )
-    {
-        p += sprintf( p, "<tr>\n" );
-        p += sprintf( p, "<td>%s</td>\n", inet_ntoa( p_con->sock.sin_addr ) );
-        p += sprintf( p, "<td>%s</td>\n", p_con->psz_file );
-        p += sprintf( p, "<td>%d</td>\n", p_con->i_http_error );
-        p += sprintf( p, "</tr>\n" );
-    }
-    p += sprintf( p, "</table>\n" );
-
-
-    /* www.videolan.org */
-    p += sprintf( p, "<hr />\n" );
-    p += sprintf( p, "<a href=\"http://www.videolan.org\">VideoLAN</a>\n" );
-    p += sprintf( p, "</body>\n" );
-    p += sprintf( p, "</html>\n" );
-
-    *pi_data = strlen( *pp_data ) + 1;
-
-    return VLC_SUCCESS;
+    return NULL;
 }
 
+static int httpd_UnbanIP( httpd_sys_t *p_httpt, httpd_banned_ip_t *p_banned_ip )
+{
+    if(!p_banned_ip)
+    {
+        return -1;
+    }
+
+    msg_Dbg( p_httpt, "Unbanning IP %s",p_banned_ip->psz_ip);
+
+    /* first cut out from list */
+    if( p_banned_ip->p_prev )
+    {
+        p_banned_ip->p_prev->p_next = p_banned_ip->p_next;
+    }
+    else
+    {
+        p_httpt->p_first_banned_ip = p_banned_ip->p_next;
+    }
+
+    if( p_banned_ip->p_next )
+    {
+        p_banned_ip->p_next->p_prev = p_banned_ip->p_prev;
+    }
+
+    FREE( p_banned_ip->psz_ip );
+    FREE( p_banned_ip );
+
+    p_httpt->i_banned_ip_count--;
+
+    return 0;
+}
 
 static void httpd_ConnnectionNew( httpd_sys_t *p_httpt, int fd, struct sockaddr_in *p_sock )
 {
@@ -1061,6 +1240,14 @@ static void httpd_ConnnectionNew( httpd_sys_t *p_httpt, int fd, struct sockaddr_
 
     msg_Dbg( p_httpt, "new connection from %s", inet_ntoa( p_sock->sin_addr ) );
 
+    /* verify if it's a banned ip */
+    if(httpd_GetbannedIP( p_httpt,inet_ntoa( p_sock->sin_addr ) ) )
+    {
+        msg_Dbg( p_httpt, "Ip %s banned : closing connection", inet_ntoa( p_sock->sin_addr ) );
+        close(fd);
+        return;
+    }
+
     /* create a new connection and link it */
     p_con = malloc( sizeof( httpd_connection_t ) );
     p_con->i_state  = HTTPD_CONNECTION_RECEIVING_REQUEST;
@@ -1074,6 +1261,9 @@ static void httpd_ConnnectionNew( httpd_sys_t *p_httpt, int fd, struct sockaddr_
     p_con->psz_password = NULL;
     p_con->p_file   = NULL;
 
+    p_con->i_request_size = 0;
+    p_con->p_request = NULL;
+
     p_con->i_buffer = 0;
     p_con->i_buffer_size = 8096;
     p_con->p_buffer = malloc( p_con->i_buffer_size );
@@ -1130,10 +1320,10 @@ static void httpd_ConnnectionClose( httpd_sys_t *p_httpt, httpd_connection_t *p_
     FREE( p_con->p_buffer );
     SOCKET_CLOSE( p_con->fd );
 
-
     FREE( p_con->psz_user );
     FREE( p_con->psz_password );
 
+    FREE( p_con->p_request );
     free( p_con );
 }
 
@@ -1251,6 +1441,7 @@ static void httpd_ConnectionParseRequest( httpd_sys_t *p_httpt, httpd_connection
 
     //msg_Dbg( p_httpt, "new request=\n%s", p_con->p_buffer );
 
+
     p = p_con->p_buffer;
     p_end = p + strlen( p ) + 1;
 
@@ -1259,10 +1450,25 @@ static void httpd_ConnectionParseRequest( httpd_sys_t *p_httpt, httpd_connection
     httpd_RequestGetWord( version, 32, &p, p_end );
     //msg_Dbg( p_httpt, "ask =%s= =%s= =%s=", command, url, version );
 
-    if( strcmp( command, "GET" ) )
+    p_con->p_request      = NULL;
+    p_con->i_request_size = 0;
+    if( !strcmp( command, "GET" ) )
+    {
+        p_con->i_method = HTTPD_CONNECTION_METHOD_GET;
+    }
+    else if( !strcmp( command, "POST" ))
+    {
+        p_con->i_method = HTTPD_CONNECTION_METHOD_POST;
+    }
+    else if( !strcmp( command, "HEAD" ))
+    {
+        p_con->i_method = HTTPD_CONNECTION_METHOD_HEAD;
+    }
+    else
     {
         /* unimplemented */
         p_con->psz_file = strdup( "/501.html" );
+        p_con->i_method = HTTPD_CONNECTION_METHOD_GET;
         p_con->i_http_error = 501;
         goto create_header;
     }
@@ -1288,6 +1494,10 @@ static void httpd_ConnectionParseRequest( httpd_sys_t *p_httpt, httpd_connection
         //msg_Dbg( p_httpt, "new line=%s", p );
 
         httpd_RequestGetWord( header, 1024, &p, p_end );
+        if( !strcmp( header, "\r\n" ) || !strcmp( header, "\n" ) )
+        {
+            break;
+        }
 
         if( !strcmp( header, "Authorization:" ) )
         {
@@ -1300,10 +1510,9 @@ static void httpd_ConnectionParseRequest( httpd_sys_t *p_httpt, httpd_connection
                 char decoded[1024];
 
                 httpd_RequestGetWord( basic, 1024, &p, p_end );
-                //msg_Dbg( p_httpt, "Authorization: basic:%s", basic );
+//                msg_Dbg( p_httpt, "Authorization: basic:%s", basic );
                 b64_decode( decoded, basic );
-
-                //msg_Dbg( p_httpt, "Authorization: decoded:%s", decoded );
+//                msg_Dbg( p_httpt, "Authorization: decoded:%s", decoded );
                 if( strchr( decoded, ':' ) )
                 {
                     char *p = strchr( decoded, ':' );
@@ -1316,7 +1525,47 @@ static void httpd_ConnectionParseRequest( httpd_sys_t *p_httpt, httpd_connection
         }
     }
 
-    p_con->psz_file = strdup( url );
+    if( strchr( url, '?' ) )
+    {
+        char *p_request = strchr( url, '?' );
+        *p_request++ = '\0';
+        p_con->psz_file = strdup( url );
+        p_con->p_request = strdup( p_request );
+        p_con->i_request_size = strlen( p_con->p_request );
+    }
+    else
+    {
+        p_con->psz_file = strdup( url );
+    }
+
+    /* fix p_request */
+    if( p_con->i_method == HTTPD_CONNECTION_METHOD_POST )
+    {
+        char *p_request;
+        if( strstr( p_con->p_buffer, "\r\n\r\n" ) )
+        {
+            p_request = strstr( p_con->p_buffer, "\r\n\r\n" ) + 4;
+        }
+        else if( strstr( p_con->p_buffer, "\n\n" ) )
+        {
+            p_request = strstr( p_con->p_buffer, "\n\n" ) + 2;
+        }
+        else
+        {
+            p_request = NULL;
+        }
+        if( p_request && p_request < p_end )
+        {
+            p_con->i_request_size = p_end - p_request;
+            p_con->p_request = malloc( p_con->i_request_size + 1);
+
+            memcpy( p_con->p_request,
+                    p_request,
+                    p_con->i_request_size );
+
+            p_con->p_request[p_con->i_request_size] = '\0';
+        }
+    }
     p_con->i_http_error = 200;
 
 create_header:
@@ -1328,11 +1577,19 @@ create_header:
     //vlc_mutex_lock( &p_httpt->file_lock );
 search_file:
     /* search file */
-    for( i = 0, p_con->p_file = NULL; i < p_httpt->i_file_count; i++ )
+    p_con->p_file = NULL;
+    for( i = 0; i < p_httpt->i_file_count; i++ )
     {
         if( !strcmp( p_httpt->file[i]->psz_file, p_con->psz_file ) )
         {
-            p_con->p_file = p_httpt->file[i];
+            if( p_httpt->file[i]->b_stream ||
+                p_con->i_method == HTTPD_CONNECTION_METHOD_HEAD ||
+                ( p_con->i_method == HTTPD_CONNECTION_METHOD_GET  && p_httpt->file[i]->pf_get ) ||
+                ( p_con->i_method == HTTPD_CONNECTION_METHOD_POST && p_httpt->file[i]->pf_post ) )
+            {
+                p_con->p_file = p_httpt->file[i];
+                break;
+            }
         }
     }
 
@@ -1377,14 +1634,15 @@ search_file:
 
     p_con->i_state = HTTPD_CONNECTION_SENDING_HEADER;
 
+    p_con->i_buffer_size = 4096;
+    p_con->i_buffer = 0;
+
     /* we send stream header with this one */
     if( p_con->i_http_error == 200 && p_con->p_file->b_stream )
     {
-        p_con->i_buffer_size = 4096 + p_con->p_file->i_header_size;
+        p_con->i_buffer_size += p_con->p_file->i_header_size;
     }
 
-    p_con->i_buffer_size = 4096;
-    p_con->i_buffer = 0;
     p = p_con->p_buffer = malloc( p_con->i_buffer_size );
 
     p += sprintf( p, "HTTP/1.0 %d %s\r\n", p_con->i_http_error, psz_status );
@@ -1393,6 +1651,7 @@ search_file:
     {
         p += sprintf( p, "WWW-Authenticate: Basic realm=\"%s\"\r\n", user );
     }
+    p += sprintf( p, "Cache-Control: no-cache\r\n" );
     p += sprintf( p, "\r\n" );
 
     p_con->i_buffer_size = strlen( p_con->p_buffer );// + 1;
@@ -1411,7 +1670,6 @@ search_file:
 #define HTTPD_STREAM_PACKET 10000
 static void httpd_Thread( httpd_sys_t *p_httpt )
 {
-    httpd_file_t    *p_page_admin;
     httpd_file_t    *p_page_401;
     httpd_file_t    *p_page_404;
 
@@ -1422,18 +1680,15 @@ static void httpd_Thread( httpd_sys_t *p_httpt )
     p_page_401 = _RegisterFile( p_httpt,
                                 "/401.html", "text/html",
                                 NULL, NULL,
-                                httpd_page_401_fill,
+                                httpd_page_401_get,
+                                NULL,
                                 (httpd_file_callback_args_t*)NULL );
     p_page_404 = _RegisterFile( p_httpt,
                                 "/404.html", "text/html",
                                 NULL, NULL,
-                                httpd_page_404_fill,
+                                httpd_page_404_get,
+                                NULL,
                                 (httpd_file_callback_args_t*)NULL );
-    p_page_admin = _RegisterFile( p_httpt,
-                                  "/admin.html", "text/html",
-                                  "admin", "salut",
-                                  httpd_page_admin_fill,
-                                  (httpd_file_callback_args_t*)p_httpt );
 
     while( !p_httpt->b_die )
     {
@@ -1463,7 +1718,8 @@ static void httpd_Thread( httpd_sys_t *p_httpt )
         for( p_con = p_httpt->p_first_connection; p_con != NULL; )
         {
             /* no more than 10s of inactivity */
-            if( p_con->i_last_activity_date + (mtime_t)HTTPD_CONNECTION_MAX_UNUSED < mdate() )
+            if( p_con->i_last_activity_date + (mtime_t)HTTPD_CONNECTION_MAX_UNUSED < mdate() ||
+                p_con->i_state == HTTPD_CONNECTION_TO_BE_CLOSED)
             {
                 httpd_connection_t *p_next = p_con->p_next;
 
@@ -1498,7 +1754,7 @@ static void httpd_Thread( httpd_sys_t *p_httpt )
         timeout.tv_sec = 0;
         timeout.tv_usec = 500*1000;
 
-        i_ret = select( i_handle_max + 1, 
+        i_ret = select( i_handle_max + 1,
                         &fds_read,
                         &fds_write,
                         NULL,
@@ -1509,9 +1765,9 @@ static void httpd_Thread( httpd_sys_t *p_httpt )
             msleep( 1000 );
             continue;
         }
+
         if( i_ret <= 0 )
         {
-//            msg_Dbg( p_httpt, "waiting..." );
             continue;
         }
 
@@ -1561,8 +1817,12 @@ static void httpd_Thread( httpd_sys_t *p_httpt )
                               p_con->p_buffer + p_con->i_buffer,
                               p_con->i_buffer_size - p_con->i_buffer, 0 );
 
-                if( ( i_len < 0 && errno != EAGAIN && errno != EINTR )||
-                    ( i_len == 0 ) )
+
+#if defined( WIN32 ) || defined( UNDER_CE )
+                if( ( i_len < 0 && WSAGetLastError() != WSAEWOULDBLOCK ) || ( i_len == 0 ) )
+#else
+                if( ( i_len < 0 && errno != EAGAIN && errno != EINTR ) || ( i_len == 0 ) )
+#endif
                 {
                     httpd_connection_t *p_next = p_con->p_next;
 
@@ -1598,12 +1858,21 @@ static void httpd_Thread( httpd_sys_t *p_httpt )
                 int i_len;
 
                 /* write data */
-                i_len = send( p_con->fd, p_con->p_buffer + p_con->i_buffer, p_con->i_buffer_size - p_con->i_buffer, 0 );
-
+                if( p_con->i_buffer_size - p_con->i_buffer > 0 )
+                {
+                    i_len = send( p_con->fd, p_con->p_buffer + p_con->i_buffer, p_con->i_buffer_size - p_con->i_buffer, 0 );
+                }
+                else
+                {
+                    i_len = 0;
+                }
 //                msg_Warn( p_httpt, "on %d send %d bytes %s", p_con->i_buffer_size, i_len, p_con->p_buffer + p_con->i_buffer );
 
-                if( ( i_len < 0 && errno != EAGAIN && errno != EINTR )||
-                    ( i_len == 0 ) )
+#if defined( WIN32 ) || defined( UNDER_CE )
+                if( ( i_len < 0 && WSAGetLastError() != WSAEWOULDBLOCK ) || ( i_len == 0 ) )
+#else
+                if( ( i_len < 0 && errno != EAGAIN && errno != EINTR ) || ( i_len == 0 ) )
+#endif
                 {
                     httpd_connection_t *p_next = p_con->p_next;
 
@@ -1623,10 +1892,27 @@ static void httpd_Thread( httpd_sys_t *p_httpt )
                             p_con->i_buffer = 0;
                             FREE( p_con->p_buffer );
 
-                            if( !p_con->p_file->b_stream )
+                            if( !p_con->p_file->b_stream || p_con->i_method == HTTPD_CONNECTION_METHOD_HEAD )
                             {
                                 p_con->i_state = HTTPD_CONNECTION_SENDING_FILE; // be sure to out from HTTPD_CONNECTION_SENDING_HEADER
-                                p_con->p_file->pf_fill( p_con->p_file->p_sys, &p_con->p_buffer, &p_con->i_buffer_size );
+                                if( p_con->i_method == HTTPD_CONNECTION_METHOD_GET )
+                                {
+                                    p_con->p_file->pf_get( p_con->p_file->p_sys,
+                                                           p_con->p_request, p_con->i_request_size,
+                                                           &p_con->p_buffer, &p_con->i_buffer_size );
+                                }
+                                else if( p_con->i_method == HTTPD_CONNECTION_METHOD_POST )
+                                {
+                                    p_con->p_file->pf_post( p_con->p_file->p_sys,
+                                                            p_con->p_request, p_con->i_request_size,
+                                                            &p_con->p_buffer, &p_con->i_buffer_size );
+                                }
+                                else
+                                {
+                                    /* HTTPD_CONNECTION_METHOD_HEAD for example */
+                                    p_con->p_buffer = NULL;
+                                    p_con->i_buffer_size = 0;
+                                }
                             }
                             else
                             {
@@ -1666,7 +1952,8 @@ static void httpd_Thread( httpd_sys_t *p_httpt )
                     /* check if this p_con aren't to late */
                     if( p_con->i_stream_pos + p_stream->i_buffer_size < p_stream->i_buffer_pos )
                     {
-                        fprintf(stderr, "fixing i_stream_pos (old=%lld i_buffer_pos=%lld\n", p_con->i_stream_pos, p_stream->i_buffer_pos  );
+                        fprintf( stderr, "fixing i_stream_pos (old=%lld i_buffer_pos=%lld\n",
+                                 p_con->i_stream_pos, p_stream->i_buffer_pos  );
                         p_con->i_stream_pos = p_stream->i_buffer_last_pos;
                     }
 
@@ -1685,7 +1972,11 @@ static void httpd_Thread( httpd_sys_t *p_httpt )
                     }
                     i_send = send( p_con->fd, &p_stream->p_buffer[i_pos], i_write, 0 );
 
+#if defined( WIN32 ) || defined( UNDER_CE )
+                    if( ( i_send < 0 && WSAGetLastError() != WSAEWOULDBLOCK )|| ( i_send == 0 ) )
+#else
                     if( ( i_send < 0 && errno != EAGAIN && errno != EINTR )|| ( i_send == 0 ) )
+#endif
                     {
                         httpd_connection_t *p_next = p_con->p_next;
 
@@ -1702,7 +1993,7 @@ static void httpd_Thread( httpd_sys_t *p_httpt )
                 p_con = p_con->p_next;
                 continue;   /* just for clarity */
             }
-            else
+            else if( p_con->i_state != HTTPD_CONNECTION_TO_BE_CLOSED )
             {
                 msg_Warn( p_httpt, "cannot occur (Invalid p_con->i_state)" );
                 p_con = p_con->p_next;
@@ -1711,9 +2002,9 @@ static void httpd_Thread( httpd_sys_t *p_httpt )
 
         vlc_mutex_unlock( &p_httpt->file_lock );
     }
+
     msg_Info( p_httpt, "httpd stopped" );
 
     _UnregisterFile( p_httpt, p_page_401 );
     _UnregisterFile( p_httpt, p_page_404 );
-    _UnregisterFile( p_httpt, p_page_admin );
 }
