@@ -2,7 +2,7 @@
  * a52.c
  *****************************************************************************
  * Copyright (C) 2001, 2002 VideoLAN
- * $Id: a52.c,v 1.3 2003/03/31 03:46:11 fenrir Exp $
+ * $Id: a52.c,v 1.5 2003/05/03 02:09:41 fenrir Exp $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Eric Petit <titer@videolan.org>
@@ -45,7 +45,7 @@ typedef struct packetizer_s
 
     /* Output properties */
     sout_packetizer_input_t *p_sout_input;
-    sout_packet_format_t    output_format;
+    sout_format_t           output_format;
 
     uint64_t                i_samplescount;
 
@@ -133,8 +133,6 @@ static int Run( decoder_fifo_t *p_fifo )
 
     EndThread( p_pack );
 
-    FREE( p_pack );
-
     if( b_error )
     {
         return( -1 );
@@ -155,8 +153,6 @@ static int InitThread( packetizer_t *p_pack )
     p_pack->output_format.i_cat = AUDIO_ES;
     p_pack->output_format.i_fourcc = VLC_FOURCC( 'a', '5', '2', ' ' );
 
-    p_pack->output_format.p_format = NULL;
-
     p_pack->p_sout_input   = NULL;
 
     p_pack->i_samplescount = 0;
@@ -230,19 +226,12 @@ static void PacketizeThread( packetizer_t *p_pack )
 
     if( !p_pack->p_sout_input )
     {
-        /* add a input for the stream ouput */
-        WAVEFORMATEX    *p_wf;
-
-        p_wf = malloc( sizeof( WAVEFORMATEX ) );
-        p_pack->output_format.p_format = (void*)p_wf;
-
-        p_wf->wFormatTag        = WAVE_FORMAT_A52;
-        p_wf->nChannels         = i_channels;
-        p_wf->nSamplesPerSec    = i_samplerate;
-        p_wf->nAvgBytesPerSec   = 0;
-        p_wf->nBlockAlign       = 1;
-        p_wf->wBitsPerSample    = 0;
-        p_wf->cbSize            = 0;
+        p_pack->output_format.i_sample_rate = i_samplerate;
+        p_pack->output_format.i_channels    = i_channels;
+        p_pack->output_format.i_block_align = 0;
+        p_pack->output_format.i_bitrate     = 0;
+        p_pack->output_format.i_extra_data  = 0;
+        p_pack->output_format.p_extra_data  = NULL;
 
         p_pack->p_sout_input =
             sout_InputNew( p_pack->p_fifo,
@@ -315,6 +304,7 @@ static void EndThread ( packetizer_t *p_pack)
     {
         sout_InputDelete( p_pack->p_sout_input );
     }
+    free( p_pack );
 }
 
 /*****************************************************************************
@@ -334,6 +324,7 @@ static int SyncInfo( const byte_t * p_buf, int * pi_channels,
                                 512, 576, 640 };
     static const uint8_t lfeon[8] = { 0x10, 0x10, 0x04, 0x04,
                                       0x04, 0x01, 0x04, 0x01 };
+    static const int acmod_to_channels[8] = { 2, 1, 2, 3, 3, 4, 4, 5 };
     int frmsizecod;
     int bitrate;
     int half;
@@ -355,36 +346,9 @@ static int SyncInfo( const byte_t * p_buf, int * pi_channels,
                         | AOUT_CHAN_DOLBYSTEREO;*/
         *pi_channels = 2; /* FIXME ???  */
     }
-    else switch ( acmod )
+    else
     {
-    case 0x0:
-        /* Dual-mono = stereo + dual-mono */
-//        *pi_channels = AOUT_CHAN_LEFT | AOUT_CHAN_RIGHT
-//                        | AOUT_CHAN_DUALMONO;
-        *pi_channels = 2; /* FIXME ???  */
-        break;
-    case 0x1:
-        /* Mono */
-        *pi_channels = 1;
-        break;
-    case 0x2:
-        /* Stereo */
-        *pi_channels = 2;
-        break;
-    case 0x3: /* 3F */
-    case 0x4: /* 2F1R */
-        *pi_channels = 3;
-        break;
-    case 0x5: /* 3F1R */
-    case 0x6: /* 2F2R */
-        *pi_channels = 4;
-        break;
-    case 0x7:
-        /* 3F2R */
-        *pi_channels = 5;
-        break;
-    default:
-        return 0;
+        *pi_channels = acmod_to_channels[acmod&0x07];
     }
 
     if ( p_buf[6] & lfeon[acmod] ) *pi_channels += 1;//|= AOUT_CHAN_LFEA;
