@@ -2,7 +2,7 @@
  * xcommon.c: Functions common to the X11 and XVideo plugins
  *****************************************************************************
  * Copyright (C) 1998-2001 VideoLAN
- * $Id: xcommon.c,v 1.14 2003/02/09 23:42:06 sigmunau Exp $
+ * $Id: xcommon.c,v 1.29 2003/08/13 18:39:52 gbazin Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
@@ -66,7 +66,9 @@
 #   include <X11/extensions/Xvlib.h>
 #endif
 
-#include "netutils.h"                                 /* network_ChannelJoin */
+#ifdef HAVE_XINERAMA
+#   include <X11/extensions/Xinerama.h>
+#endif
 
 #include "xcommon.h"
 
@@ -115,6 +117,8 @@ static void SetPalette     ( vout_thread_t *,
                              uint16_t *, uint16_t *, uint16_t * );
 #endif
 
+static void TestNetWMSupport( vout_thread_t * );
+
 /*****************************************************************************
  * Activate: allocate X11 video thread output method
  *****************************************************************************
@@ -171,11 +175,9 @@ int E_(Activate) ( vlc_object_t *p_this )
     {
         if( strlen( psz_chroma ) >= 4 )
         {
-            i_chroma  = (unsigned char)psz_chroma[0] <<  0;
-            i_chroma |= (unsigned char)psz_chroma[1] <<  8;
-            i_chroma |= (unsigned char)psz_chroma[2] << 16;
-            i_chroma |= (unsigned char)psz_chroma[3] << 24;
-
+            /* Do not use direct assignment because we are not sure of the
+             * alignment. */
+            memcpy(&i_chroma, psz_chroma, 4);
             b_chroma = 1;
         }
 
@@ -193,7 +195,7 @@ int E_(Activate) ( vlc_object_t *p_this )
     }
 
     /* Check that we have access to an XVideo port providing this chroma */
-    p_vout->p_sys->i_xvport = XVideoGetPort( p_vout, i_chroma,
+    p_vout->p_sys->i_xvport = XVideoGetPort( p_vout, VLC2X11_FOURCC(i_chroma),
                                              &p_vout->output.i_chroma );
     if( p_vout->p_sys->i_xvport < 0 )
     {
@@ -210,7 +212,7 @@ int E_(Activate) ( vlc_object_t *p_this )
          * XVideo port for an YUY2 picture. We'll need to do an YUV
          * conversion, but at least it has got scaling. */
         p_vout->p_sys->i_xvport =
-                        XVideoGetPort( p_vout, VLC_FOURCC('Y','U','Y','2'),
+                        XVideoGetPort( p_vout, X11_FOURCC('Y','U','Y','2'),
                                                &p_vout->output.i_chroma );
         if( p_vout->p_sys->i_xvport < 0 )
         {
@@ -218,7 +220,7 @@ int E_(Activate) ( vlc_object_t *p_this )
              * XVideo port for a simple 16bpp RGB picture. We'll need to do
              * an YUV conversion, but at least it has got scaling. */
             p_vout->p_sys->i_xvport =
-                            XVideoGetPort( p_vout, VLC_FOURCC('R','V','1','6'),
+                            XVideoGetPort( p_vout, X11_FOURCC('R','V','1','6'),
                                                    &p_vout->output.i_chroma );
             if( p_vout->p_sys->i_xvport < 0 )
             {
@@ -228,6 +230,7 @@ int E_(Activate) ( vlc_object_t *p_this )
             }
         }
     }
+    p_vout->output.i_chroma = X112VLC_FOURCC(p_vout->output.i_chroma);
 #endif
 
     /* Create blank cursor (for mouse cursor autohiding) */
@@ -268,6 +271,8 @@ int E_(Activate) ( vlc_object_t *p_this )
     p_vout->p_sys->b_altfullscreen = 0;
     p_vout->p_sys->i_time_button_last_pressed = 0;
 
+    TestNetWMSupport( p_vout );
+
     return VLC_SUCCESS;
 }
 
@@ -524,12 +529,23 @@ static int ManageVideo( vout_thread_t *p_vout )
                 }
                 else
                 {
-                    p_vout->p_vlc->b_die = 1;
+                    /* the user wants to close the window */
+                    playlist_t * p_playlist =
+                        (playlist_t *)vlc_object_find( p_vout,
+                                                       VLC_OBJECT_PLAYLIST,
+                                                       FIND_ANYWHERE );
+                    if( p_playlist != NULL )
+                    {
+                        playlist_Stop( p_playlist );
+                        vlc_object_release( p_playlist );
+                    }
                 }
                 break;
             case XK_Menu:
                 {
                     intf_thread_t *p_intf;
+                    playlist_t * p_playlist;
+
                     p_intf = vlc_object_find( p_vout, VLC_OBJECT_INTF,
                                                       FIND_ANYWHERE );
                     if( p_intf )
@@ -537,19 +553,41 @@ static int ManageVideo( vout_thread_t *p_vout )
                         p_intf->b_menu_change = 1;
                         vlc_object_release( p_intf );
                     }
+
+                    p_playlist = vlc_object_find( p_vout, VLC_OBJECT_PLAYLIST,
+                                                           FIND_ANYWHERE );
+                    if( p_playlist != NULL )
+                    {
+                        vlc_value_t val;
+                        var_Set( p_playlist, "intf-popupmenu", val );
+                        vlc_object_release( p_playlist );
+                    }
                 }
                 break;
             case XK_Left:
-                input_Seek( p_vout, -5, INPUT_SEEK_SECONDS | INPUT_SEEK_CUR );
+/*                input_Seek( p_vout, -5, INPUT_SEEK_SECONDS | INPUT_SEEK_CUR ); */
+                val.psz_string = "LEFT";
+                var_Set( p_vout, "key-pressed", val );
                 break;
             case XK_Right:
-                input_Seek( p_vout, 5, INPUT_SEEK_SECONDS | INPUT_SEEK_CUR );
+/*                input_Seek( p_vout, 5, INPUT_SEEK_SECONDS | INPUT_SEEK_CUR ); */
+                val.psz_string = "RIGHT";
+                var_Set( p_vout, "key-pressed", val );
                 break;
             case XK_Up:
-                input_Seek( p_vout, 60, INPUT_SEEK_SECONDS | INPUT_SEEK_CUR );
+/*                input_Seek( p_vout, 60, INPUT_SEEK_SECONDS | INPUT_SEEK_CUR ); */
+                val.psz_string = "UP";
+                var_Set( p_vout, "key-pressed", val );
                 break;
             case XK_Down:
-                input_Seek( p_vout, -60, INPUT_SEEK_SECONDS | INPUT_SEEK_CUR );
+/*                input_Seek( p_vout, -60, INPUT_SEEK_SECONDS | INPUT_SEEK_CUR ); */
+                val.psz_string = "DOWN";
+                var_Set( p_vout, "key-pressed", val );
+                break;
+            case XK_Return:
+            case XK_KP_Enter:
+                val.psz_string = "ENTER";
+                var_Set( p_vout, "key-pressed", val );
                 break;
             case XK_Home:
                 input_Seek( p_vout, 0, INPUT_SEEK_BYTES | INPUT_SEEK_SET );
@@ -558,28 +596,15 @@ static int ManageVideo( vout_thread_t *p_vout )
                 input_Seek( p_vout, 0, INPUT_SEEK_BYTES | INPUT_SEEK_END );
                 break;
             case XK_Page_Up:
-                input_Seek( p_vout, 900, INPUT_SEEK_SECONDS | INPUT_SEEK_CUR );
+                input_Seek( p_vout, 10, INPUT_SEEK_SECONDS | INPUT_SEEK_CUR );
                 break;
             case XK_Page_Down:
-                input_Seek( p_vout, -900, INPUT_SEEK_SECONDS | INPUT_SEEK_CUR );
+                input_Seek( p_vout, -10, INPUT_SEEK_SECONDS | INPUT_SEEK_CUR );
                 break;
             case XK_space:
                 input_SetStatus( p_vout, INPUT_STATUS_PAUSE );
                 break;
 
-            case XK_F1: network_ChannelJoin( p_vout, 1 ); break;
-            case XK_F2: network_ChannelJoin( p_vout, 2 ); break;
-            case XK_F3: network_ChannelJoin( p_vout, 3 ); break;
-            case XK_F4: network_ChannelJoin( p_vout, 4 ); break;
-            case XK_F5: network_ChannelJoin( p_vout, 5 ); break;
-            case XK_F6: network_ChannelJoin( p_vout, 6 ); break;
-            case XK_F7: network_ChannelJoin( p_vout, 7 ); break;
-            case XK_F8: network_ChannelJoin( p_vout, 8 ); break;
-            case XK_F9: network_ChannelJoin( p_vout, 9 ); break;
-            case XK_F10: network_ChannelJoin( p_vout, 10 ); break;
-            case XK_F11: network_ChannelJoin( p_vout, 11 ); break;
-            case XK_F12: network_ChannelJoin( p_vout, 12 ); break;
-
             default:
                 /* "Normal Keys"
                  * The reason why I use this instead of XK_0 is that
@@ -669,14 +694,32 @@ static int ManageVideo( vout_thread_t *p_vout )
                     break;
                     
                 case Button2:
-                    var_Get( p_vout, "mouse-button-down", &val );
-                    val.i_int &= ~2;
-                    var_Set( p_vout, "mouse-button-down", val );
+                    {
+                        playlist_t *p_playlist;
+
+                        var_Get( p_vout, "mouse-button-down", &val );
+                        val.i_int &= ~2;
+                        var_Set( p_vout, "mouse-button-down", val );
+
+                        p_playlist = vlc_object_find( p_vout,
+                                                      VLC_OBJECT_PLAYLIST,
+                                                      FIND_ANYWHERE );
+                        if( p_playlist != NULL )
+                        {
+                            vlc_value_t val;
+                            var_Get( p_playlist, "intf-show", &val );
+                            val.b_bool = !val.b_bool;
+                            var_Set( p_playlist, "intf-show", val );
+                            vlc_object_release( p_playlist );
+                        }
+                    }
                     break;
                     
                 case Button3:
                     {
                         intf_thread_t *p_intf;
+                        playlist_t *p_playlist;
+
                         var_Get( p_vout, "mouse-button-down", &val );
                         val.i_int &= ~4;
                         var_Set( p_vout, "mouse-button-down", val );
@@ -687,6 +730,16 @@ static int ManageVideo( vout_thread_t *p_vout )
                             p_intf->b_menu_change = 1;
                             vlc_object_release( p_intf );
                         }
+
+                        p_playlist = vlc_object_find( p_vout,
+                                                      VLC_OBJECT_PLAYLIST,
+                                                      FIND_ANYWHERE );
+                        if( p_playlist != NULL )
+                        {
+                            vlc_value_t val;
+                            var_Set( p_playlist, "intf-popupmenu", val );
+                            vlc_object_release( p_playlist );
+                        }
                     }
                     break;
 
@@ -780,7 +833,15 @@ static int ManageVideo( vout_thread_t *p_vout )
                && ((Atom)xevent.xclient.data.l[0]
                      == p_vout->p_sys->p_win->wm_delete_window ) )
         {
-            p_vout->p_vlc->b_die = 1;
+            /* the user wants to close the window */
+            playlist_t * p_playlist =
+                (playlist_t *)vlc_object_find( p_vout, VLC_OBJECT_PLAYLIST,
+                                               FIND_ANYWHERE );
+            if( p_playlist != NULL )
+            {
+                playlist_Stop( p_playlist );
+                vlc_object_release( p_playlist );
+            }
         }
     }
 
@@ -789,6 +850,12 @@ static int ManageVideo( vout_thread_t *p_vout )
      */
     if ( p_vout->i_changes & VOUT_FULLSCREEN_CHANGE )
     {
+        vlc_value_t val;
+
+        /* Update the object variable and trigger callback */
+        val.b_bool = !p_vout->b_fullscreen;
+        var_Set( p_vout, "fullscreen", val );
+
         ToggleFullScreen( p_vout );
         p_vout->i_changes &= ~VOUT_FULLSCREEN_CHANGE;
     }
@@ -865,9 +932,9 @@ static int CreateWindow( vout_thread_t *p_vout, x11_window_t *p_win )
     XGCValues               xgcvalues;
     XEvent                  xevent;
 
-    vlc_bool_t              b_expose;
-    vlc_bool_t              b_configure_notify;
-    vlc_bool_t              b_map_notify;
+    vlc_bool_t              b_expose = VLC_FALSE;
+    vlc_bool_t              b_configure_notify = VLC_FALSE;
+    vlc_bool_t              b_map_notify = VLC_FALSE;
 
     vlc_value_t             val;
     long long int           i_drawable;
@@ -893,8 +960,6 @@ static int CreateWindow( vout_thread_t *p_vout, x11_window_t *p_win )
 
     if( !i_drawable )
     {
-        p_win->b_owned = VLC_TRUE;
-
         /* Create the window and set hints - the window must receive
          * ConfigureNotify events, and until it is displayed, Expose and
          * MapNotify events. */
@@ -929,13 +994,29 @@ static int CreateWindow( vout_thread_t *p_vout, x11_window_t *p_win )
     }
     else
     {
-        p_win->b_owned = VLC_FALSE;
-        p_win->base_window = i_drawable;
+        /* Get the parent window's geometry information */
+        Window dummy1;
+        unsigned int dummy2, dummy3;
+        XGetGeometry( p_vout->p_sys->p_display, i_drawable,
+                      &dummy1, &dummy2, &dummy3,
+                      &p_win->i_width,
+                      &p_win->i_height,
+                      &dummy2, &dummy3 );
 
-        XChangeWindowAttributes( p_vout->p_sys->p_display,
-                                 p_win->base_window,
-                                 CWBackingStore | CWBackPixel | CWEventMask,
-                                 &xwindow_attributes );
+        /* We are already configured */
+        b_configure_notify = VLC_TRUE;
+
+        /* From man XSelectInput: only one client at a time can select a
+         * ButtonPress event, so we need to open a new window anyway. */
+        p_win->base_window =
+            XCreateWindow( p_vout->p_sys->p_display,
+                           i_drawable,
+                           0, 0,
+                           p_win->i_width, p_win->i_height,
+                           0,
+                           0, CopyFromParent, 0,
+                           CWBackingStore | CWBackPixel | CWEventMask,
+                           &xwindow_attributes );
     }
 
     if( (p_win->wm_protocols == None)        /* use WM_DELETE_WINDOW */
@@ -954,50 +1035,33 @@ static int CreateWindow( vout_thread_t *p_vout, x11_window_t *p_win )
                            p_win->base_window,
                            GCGraphicsExposures, &xgcvalues );
 
-    if( p_win->b_owned )
+    /* Send orders to server, and wait until window is displayed - three
+     * events must be received: a MapNotify event, an Expose event allowing
+     * drawing in the window, and a ConfigureNotify to get the window
+     * dimensions. Once those events have been received, only
+     * ConfigureNotify events need to be received. */
+    XMapWindow( p_vout->p_sys->p_display, p_win->base_window );
+    do
     {
-        /* Send orders to server, and wait until window is displayed - three
-         * events must be received: a MapNotify event, an Expose event allowing
-         * drawing in the window, and a ConfigureNotify to get the window
-         * dimensions. Once those events have been received, only
-         * ConfigureNotify events need to be received. */
-        b_expose = 0;
-        b_configure_notify = 0;
-        b_map_notify = 0;
-        XMapWindow( p_vout->p_sys->p_display, p_win->base_window );
-        do
+        XNextEvent( p_vout->p_sys->p_display, &xevent);
+        if( (xevent.type == Expose)
+            && (xevent.xexpose.window == p_win->base_window) )
         {
-            XNextEvent( p_vout->p_sys->p_display, &xevent);
-            if( (xevent.type == Expose)
-                && (xevent.xexpose.window == p_win->base_window) )
-            {
-                b_expose = 1;
-            }
-            else if( (xevent.type == MapNotify)
-                     && (xevent.xmap.window == p_win->base_window) )
-            {
-                b_map_notify = 1;
-            }
-            else if( (xevent.type == ConfigureNotify)
-                     && (xevent.xconfigure.window == p_win->base_window) )
-            {
-                b_configure_notify = 1;
-                p_win->i_width = xevent.xconfigure.width;
-                p_win->i_height = xevent.xconfigure.height;
-            }
-        } while( !( b_expose && b_configure_notify && b_map_notify ) );
-    }
-    else
-    {
-        /* Get the window's geometry information */
-        Window dummy1;
-        unsigned int dummy2, dummy3;
-        XGetGeometry( p_vout->p_sys->p_display, p_win->base_window,
-                      &dummy1, &dummy2, &dummy3,
-                      &p_win->i_width,
-                      &p_win->i_height,
-                      &dummy2, &dummy3 );
-    }
+            b_expose = VLC_TRUE;
+        }
+        else if( (xevent.type == MapNotify)
+                 && (xevent.xmap.window == p_win->base_window) )
+        {
+            b_map_notify = VLC_TRUE;
+        }
+        else if( (xevent.type == ConfigureNotify)
+                 && (xevent.xconfigure.window == p_win->base_window) )
+        {
+            b_configure_notify = VLC_TRUE;
+            p_win->i_width = xevent.xconfigure.width;
+            p_win->i_height = xevent.xconfigure.height;
+        }
+    } while( !( b_expose && b_configure_notify && b_map_notify ) );
 
     XSelectInput( p_vout->p_sys->p_display, p_win->base_window,
                   StructureNotifyMask | KeyPressMask |
@@ -1005,8 +1069,7 @@ static int CreateWindow( vout_thread_t *p_vout, x11_window_t *p_win )
                   PointerMotionMask );
 
 #ifdef MODULE_NAME_IS_x11
-    if( p_win->b_owned &&
-         XDefaultDepth(p_vout->p_sys->p_display, p_vout->p_sys->i_screen) == 8 )
+    if( XDefaultDepth(p_vout->p_sys->p_display, p_vout->p_sys->i_screen) == 8 )
     {
         /* Allocate a new palette */
         p_vout->p_sys->colormap =
@@ -1074,11 +1137,8 @@ static void DestroyWindow( vout_thread_t *p_vout, x11_window_t *p_win )
     XDestroyWindow( p_vout->p_sys->p_display, p_win->video_window );
     XFreeGC( p_vout->p_sys->p_display, p_win->gc );
 
-    if( p_win->b_owned )
-    {
-        XUnmapWindow( p_vout->p_sys->p_display, p_win->base_window );
-        XDestroyWindow( p_vout->p_sys->p_display, p_win->base_window );
-    }
+    XUnmapWindow( p_vout->p_sys->p_display, p_win->base_window );
+    XDestroyWindow( p_vout->p_sys->p_display, p_win->base_window );
 }
 
 /*****************************************************************************
@@ -1344,6 +1404,10 @@ static void ToggleFullScreen ( vout_thread_t *p_vout )
     mwmhints_t mwmhints;
     XSetWindowAttributes attributes;
 
+#ifdef HAVE_XINERAMA
+    int i_d1, i_d2;
+#endif
+
     p_vout->b_fullscreen = !p_vout->b_fullscreen;
 
     if( p_vout->b_fullscreen )
@@ -1353,11 +1417,8 @@ static void ToggleFullScreen ( vout_thread_t *p_vout )
         p_vout->p_sys->b_altfullscreen =
             config_GetInt( p_vout, MODULE_STRING "-altfullscreen" );
 
-        if( p_vout->p_sys->p_win->b_owned )
-        {
-            XUnmapWindow( p_vout->p_sys->p_display,
-                          p_vout->p_sys->p_win->base_window);
-        }
+        XUnmapWindow( p_vout->p_sys->p_display,
+                      p_vout->p_sys->p_win->base_window);
 
         p_vout->p_sys->p_win = &p_vout->p_sys->fullscreen_window;
 
@@ -1396,6 +1457,32 @@ static void ToggleFullScreen ( vout_thread_t *p_vout )
                                      p_vout->p_sys->p_win->base_window,
                                      CWOverrideRedirect,
                                      &attributes);
+
+            /* Make sure the change is effective */
+            XReparentWindow( p_vout->p_sys->p_display,
+                             p_vout->p_sys->p_win->base_window,
+                             DefaultRootWindow( p_vout->p_sys->p_display ),
+                             0, 0 );
+        }
+
+        if( p_vout->p_sys->b_net_wm_state_fullscreen )
+        {
+            XClientMessageEvent event;
+
+            memset( &event, 0, sizeof( XClientMessageEvent ) );
+
+            event.type = ClientMessage;
+            event.message_type = p_vout->p_sys->net_wm_state;
+            event.display = p_vout->p_sys->p_display;
+            event.window = p_vout->p_sys->p_win->base_window;
+            event.format = 32;
+            event.data.l[ 0 ] = 1; /* set property */
+            event.data.l[ 1 ] = p_vout->p_sys->net_wm_state_fullscreen;
+
+            XSendEvent( p_vout->p_sys->p_display,
+                        DefaultRootWindow( p_vout->p_sys->p_display ),
+                        False, SubstructureRedirectMask,
+                        (XEvent*)&event );
         }
 
         /* Make sure the change is effective */
@@ -1410,9 +1497,54 @@ static void ToggleFullScreen ( vout_thread_t *p_vout )
         p_vout->p_sys->p_win->i_height =
             DisplayHeight( p_vout->p_sys->p_display, p_vout->p_sys->i_screen );
 
+        p_vout->p_sys->p_win->i_x = 0;
+        p_vout->p_sys->p_win->i_y = 0;
+
+#ifdef HAVE_XINERAMA
+        if( XineramaQueryExtension( p_vout->p_sys->p_display, &i_d1, &i_d2 ) &&
+            XineramaIsActive( p_vout->p_sys->p_display ) )
+        {
+            XineramaScreenInfo *screens;   /* infos for xinerama */
+            int i_num_screens;
+
+            msg_Dbg( p_vout, "Using XFree Xinerama extension");
+
+#define SCREEN p_vout->p_sys->p_win->i_screen
+
+            /* Get Informations about Xinerama (num of screens) */
+            screens = XineramaQueryScreens( p_vout->p_sys->p_display,
+                                            &i_num_screens );
+
+            if( !SCREEN )
+                SCREEN = config_GetInt( p_vout,
+                                        MODULE_STRING "-xineramascreen" );
+
+            /* just check that user has entered a good value */
+            if( SCREEN >= i_num_screens || SCREEN < 0 )
+            {
+                msg_Dbg( p_vout, "requested screen number invalid" );
+                SCREEN = 0;
+            }
+
+            /* Get the X/Y upper left corner coordinate of the above screen */
+            p_vout->p_sys->p_win->i_x = screens[SCREEN].x_org;
+            p_vout->p_sys->p_win->i_y = screens[SCREEN].y_org;
+
+            /* Set the Height/width to the screen resolution */
+            p_vout->p_sys->p_win->i_width = screens[SCREEN].width;
+            p_vout->p_sys->p_win->i_height = screens[SCREEN].height;
+
+            XFree(screens);
+
+#undef SCREEN
+
+        }
+#endif
+
         XMoveResizeWindow( p_vout->p_sys->p_display,
                            p_vout->p_sys->p_win->base_window,
-                           0, 0,
+                           p_vout->p_sys->p_win->i_x,
+                           p_vout->p_sys->p_win->i_y,
                            p_vout->p_sys->p_win->i_width,
                            p_vout->p_sys->p_win->i_height );
     }
@@ -1422,32 +1554,22 @@ static void ToggleFullScreen ( vout_thread_t *p_vout )
         DestroyWindow( p_vout, &p_vout->p_sys->fullscreen_window );
         p_vout->p_sys->p_win = &p_vout->p_sys->original_window;
 
-        if( p_vout->p_sys->p_win->b_owned )
-        {
-            XMapWindow( p_vout->p_sys->p_display,
-                        p_vout->p_sys->p_win->base_window);
-        }
+        XMapWindow( p_vout->p_sys->p_display,
+                    p_vout->p_sys->p_win->base_window);
     }
 
     /* Unfortunately, using XSync() here is not enough to ensure the
      * window has already been mapped because the XMapWindow() request
      * has not necessarily been sent directly to our window (remember,
      * the call is first redirected to the window manager) */
-    if( p_vout->p_sys->p_win->b_owned )
+    do
     {
-        do
-        {
-            XWindowEvent( p_vout->p_sys->p_display,
-                          p_vout->p_sys->p_win->base_window,
-                          StructureNotifyMask, &xevent );
-        } while( xevent.type != MapNotify );
-    }
-    else
-    {
-        XSync( p_vout->p_sys->p_display, False );
-    }
+        XWindowEvent( p_vout->p_sys->p_display,
+                      p_vout->p_sys->p_win->base_window,
+                      StructureNotifyMask, &xevent );
+    } while( xevent.type != MapNotify );
 
-    /* Becareful, this can generate a BadMatch error if the window is not
+    /* Be careful, this can generate a BadMatch error if the window is not
      * already mapped by the server (see above) */
     XSetInputFocus(p_vout->p_sys->p_display,
                    p_vout->p_sys->p_win->base_window,
@@ -1776,16 +1898,17 @@ static int XVideoGetPort( vout_thread_t *p_vout,
 
     if( i_selected_port == -1 )
     {
+        int i_chroma_tmp = X112VLC_FOURCC( i_chroma );
         if( i_requested_adaptor == -1 )
         {
             msg_Warn( p_vout, "no free XVideo port found for format "
-                       "0x%.8x (%4.4s)", i_chroma, (char*)&i_chroma );
+                      "0x%.8x (%4.4s)", i_chroma_tmp, (char*)&i_chroma_tmp );
         }
         else
         {
             msg_Warn( p_vout, "XVideo adaptor %i does not have a free "
-                       "XVideo port for format 0x%.8x (%4.4s)",
-                       i_requested_adaptor, i_chroma, (char*)&i_chroma );
+                      "XVideo port for format 0x%.8x (%4.4s)",
+                      i_requested_adaptor, i_chroma_tmp, (char*)&i_chroma_tmp );
         }
     }
 
@@ -2104,3 +2227,74 @@ static void SetPalette( vout_thread_t *p_vout,
                   p_vout->p_sys->colormap, p_colors, 255 );
 }
 #endif
+
+/*****************************************************************************
+ * TestNetWMSupport: tests for Extended Window Manager Hints support
+ *****************************************************************************/
+static void TestNetWMSupport( vout_thread_t *p_vout )
+{
+    int i_ret, i_format;
+    unsigned long i, i_items, i_bytesafter;
+    Atom net_wm_supported, *p_args = NULL;
+
+    p_vout->p_sys->b_net_wm_state_fullscreen = VLC_FALSE;
+    p_vout->p_sys->b_net_wm_state_above = VLC_FALSE;
+    p_vout->p_sys->b_net_wm_state_below = VLC_FALSE;
+    p_vout->p_sys->b_net_wm_state_stays_on_top = VLC_FALSE;
+
+    net_wm_supported =
+        XInternAtom( p_vout->p_sys->p_display, "_NET_SUPPORTED", False );
+
+    i_ret = XGetWindowProperty( p_vout->p_sys->p_display,
+                                DefaultRootWindow( p_vout->p_sys->p_display ),
+                                net_wm_supported,
+                                0, 16384, False, AnyPropertyType,
+                                &net_wm_supported,
+                                &i_format, &i_items, &i_bytesafter,
+                                (unsigned char **)&p_args );
+
+    if( i_ret != Success || i_items == 0 ) return;
+
+    msg_Dbg( p_vout, "Window manager supports NetWM" );
+
+    p_vout->p_sys->net_wm_state =
+        XInternAtom( p_vout->p_sys->p_display, "_NET_WM_STATE", False );
+    p_vout->p_sys->net_wm_state_fullscreen =
+        XInternAtom( p_vout->p_sys->p_display, "_NET_WM_STATE_FULLSCREEN",
+                     False );
+    p_vout->p_sys->net_wm_state_above =
+        XInternAtom( p_vout->p_sys->p_display, "_NET_WM_STATE_ABOVE", False );
+    p_vout->p_sys->net_wm_state_below =
+        XInternAtom( p_vout->p_sys->p_display, "_NET_WM_STATE_BELOW", False );
+    p_vout->p_sys->net_wm_state_stays_on_top =
+        XInternAtom( p_vout->p_sys->p_display, "_NET_WM_STATE_STAYS_ON_TOP",
+                     False );
+
+    for( i = 0; i < i_items; i++ )
+    {
+        if( p_args[i] == p_vout->p_sys->net_wm_state_fullscreen )
+        {
+            msg_Dbg( p_vout,
+                     "Window manager supports _NET_WM_STATE_FULLSCREEN" );
+            p_vout->p_sys->b_net_wm_state_fullscreen = VLC_TRUE;
+        }
+        else if( p_args[i] == p_vout->p_sys->net_wm_state_above )
+        {
+            msg_Dbg( p_vout, "Window manager supports _NET_WM_STATE_ABOVE" );
+            p_vout->p_sys->b_net_wm_state_above = VLC_TRUE;
+        }
+        else if( p_args[i] == p_vout->p_sys->net_wm_state_below )
+        {
+            msg_Dbg( p_vout, "Window manager supports _NET_WM_STATE_BELOW" );
+            p_vout->p_sys->b_net_wm_state_below = VLC_TRUE;
+        }
+        else if( p_args[i] == p_vout->p_sys->net_wm_state_stays_on_top )
+        {
+            msg_Dbg( p_vout,
+                     "Window manager supports _NET_WM_STATE_STAYS_ON_TOP" );
+            p_vout->p_sys->b_net_wm_state_stays_on_top = VLC_TRUE;
+        }
+    }
+
+    XFree( p_args );
+}
