@@ -2,7 +2,7 @@
  * configuration.c management of the modules configuration
  *****************************************************************************
  * Copyright (C) 2001 VideoLAN
- * $Id: configuration.c,v 1.50 2003/01/30 20:59:36 sam Exp $
+ * $Id: configuration.c,v 1.61 2003/08/03 23:11:21 gbazin Exp $
  *
  * Authors: Gildas Bazin <gbazin@netcourrier.com>
  *
@@ -56,6 +56,60 @@
 #   include <direct.h>
 #endif
 
+/*****************************************************************************
+ * config_GetType: get the type of a variable (bool, int, float, string)
+ *****************************************************************************
+ * This function is used to get the type of a variable from its name.
+ * Beware, this is quite slow.
+ *****************************************************************************/
+int __config_GetType( vlc_object_t *p_this, const char *psz_name )
+{
+    module_config_t *p_config;
+    int i_type;
+
+    p_config = config_FindConfig( p_this, psz_name );
+
+    /* sanity checks */
+    if( !p_config )
+    {
+        return 0;
+    }
+
+    switch( p_config->i_type )
+    {
+    case CONFIG_ITEM_BOOL:
+        i_type = VLC_VAR_BOOL;
+        break;
+
+    case CONFIG_ITEM_INTEGER:
+        i_type = VLC_VAR_INTEGER;
+        break;
+
+    case CONFIG_ITEM_FLOAT:
+        i_type = VLC_VAR_FLOAT;
+        break;
+
+    case CONFIG_ITEM_MODULE:
+    case CONFIG_ITEM_STRING:
+        i_type = VLC_VAR_STRING;
+        break;
+
+    case CONFIG_ITEM_FILE:
+        i_type = VLC_VAR_FILE;
+        break;
+
+    case CONFIG_ITEM_DIRECTORY:
+        i_type = VLC_VAR_DIRECTORY;
+        break;
+
+    default:
+        i_type = 0;
+        break;
+    }
+
+    return i_type;
+}
+
 /*****************************************************************************
  * config_GetInt: get the value of an int variable
  *****************************************************************************
@@ -117,7 +171,7 @@ float __config_GetFloat( vlc_object_t *p_this, const char *psz_name )
  *****************************************************************************
  * This function is used to get the value of variables which are internally
  * represented by a string (CONFIG_ITEM_STRING, CONFIG_ITEM_FILE,
- * and CONFIG_ITEM_MODULE).
+ * CONFIG_ITEM_DIRECTORY, and CONFIG_ITEM_MODULE).
  *
  * Important note: remember to free() the returned char* because it's a
  *   duplicate of the actual value. It isn't safe to return a pointer to the
@@ -138,6 +192,7 @@ char * __config_GetPsz( vlc_object_t *p_this, const char *psz_name )
     }
     if( (p_config->i_type!=CONFIG_ITEM_STRING) &&
         (p_config->i_type!=CONFIG_ITEM_FILE) &&
+        (p_config->i_type!=CONFIG_ITEM_DIRECTORY) &&
         (p_config->i_type!=CONFIG_ITEM_MODULE) )
     {
         msg_Err( p_this, "option %s does not refer to a string", psz_name );
@@ -157,23 +212,25 @@ char * __config_GetPsz( vlc_object_t *p_this, const char *psz_name )
  *****************************************************************************
  * This function is used to set the value of variables which are internally
  * represented by a string (CONFIG_ITEM_STRING, CONFIG_ITEM_FILE,
- * and CONFIG_ITEM_MODULE).
+ * CONFIG_ITEM_DIRECTORY, and CONFIG_ITEM_MODULE).
  *****************************************************************************/
 void __config_PutPsz( vlc_object_t *p_this,
                       const char *psz_name, const char *psz_value )
 {
     module_config_t *p_config;
+    vlc_value_t oldval, val;
 
     p_config = config_FindConfig( p_this, psz_name );
 
     /* sanity checks */
     if( !p_config )
     {
-        msg_Err( p_this, "option %s does not exist", psz_name );
+        msg_Warn( p_this, "option %s does not exist", psz_name );
         return;
     }
     if( (p_config->i_type!=CONFIG_ITEM_STRING) &&
         (p_config->i_type!=CONFIG_ITEM_FILE) &&
+        (p_config->i_type!=CONFIG_ITEM_DIRECTORY) &&
         (p_config->i_type!=CONFIG_ITEM_MODULE) )
     {
         msg_Err( p_this, "option %s does not refer to a string", psz_name );
@@ -182,18 +239,24 @@ void __config_PutPsz( vlc_object_t *p_this,
 
     vlc_mutex_lock( p_config->p_lock );
 
-    /* free old string */
-    if( p_config->psz_value ) free( p_config->psz_value );
+    /* backup old value */
+    oldval.psz_string = p_config->psz_value;
 
-    if( psz_value ) p_config->psz_value = strdup( psz_value );
+    if( psz_value && *psz_value ) p_config->psz_value = strdup( psz_value );
     else p_config->psz_value = NULL;
 
+    val.psz_string = p_config->psz_value;
+
     vlc_mutex_unlock( p_config->p_lock );
 
     if( p_config->pf_callback )
     {
-        p_config->pf_callback( p_this );
+        p_config->pf_callback( p_this, psz_name, oldval, val,
+                               p_config->p_callback_data );
     }
+
+    /* free old string */
+    if( oldval.psz_string ) free( oldval.psz_string );
 }
 
 /*****************************************************************************
@@ -206,13 +269,14 @@ void __config_PutPsz( vlc_object_t *p_this,
 void __config_PutInt( vlc_object_t *p_this, const char *psz_name, int i_value )
 {
     module_config_t *p_config;
+    vlc_value_t oldval, val;
 
     p_config = config_FindConfig( p_this, psz_name );
 
     /* sanity checks */
     if( !p_config )
     {
-        msg_Err( p_this, "option %s does not exist", psz_name );
+        msg_Warn( p_this, "option %s does not exist", psz_name );
         return;
     }
     if( (p_config->i_type!=CONFIG_ITEM_INTEGER) &&
@@ -222,6 +286,9 @@ void __config_PutInt( vlc_object_t *p_this, const char *psz_name, int i_value )
         return;
     }
 
+    /* backup old value */
+    oldval.i_int = p_config->i_value;
+
     /* if i_min == i_max == 0, then do not use them */
     if ((p_config->i_min == 0) && (p_config->i_max == 0))
     {
@@ -240,9 +307,12 @@ void __config_PutInt( vlc_object_t *p_this, const char *psz_name, int i_value )
         p_config->i_value = i_value;
     }
 
+    val.i_int = p_config->i_value;
+
     if( p_config->pf_callback )
     {
-        p_config->pf_callback( p_this );
+        p_config->pf_callback( p_this, psz_name, oldval, val,
+                               p_config->p_callback_data );
     }
 }
 
@@ -256,13 +326,14 @@ void __config_PutFloat( vlc_object_t *p_this,
                         const char *psz_name, float f_value )
 {
     module_config_t *p_config;
+    vlc_value_t oldval, val;
 
     p_config = config_FindConfig( p_this, psz_name );
 
     /* sanity checks */
     if( !p_config )
     {
-        msg_Err( p_this, "option %s does not exist", psz_name );
+        msg_Warn( p_this, "option %s does not exist", psz_name );
         return;
     }
     if( p_config->i_type != CONFIG_ITEM_FLOAT )
@@ -271,6 +342,9 @@ void __config_PutFloat( vlc_object_t *p_this,
         return;
     }
 
+    /* backup old value */
+    oldval.f_float = p_config->f_value;
+
     /* if f_min == f_max == 0, then do not use them */
     if ((p_config->f_min == 0) && (p_config->f_max == 0))
     {
@@ -289,9 +363,12 @@ void __config_PutFloat( vlc_object_t *p_this,
         p_config->f_value = f_value;
     }
 
+    val.f_float = p_config->f_value;
+
     if( p_config->pf_callback )
     {
-        p_config->pf_callback( p_this );
+        p_config->pf_callback( p_this, psz_name, oldval, val,
+                               p_config->p_callback_data );
     }
 }
 
@@ -385,23 +462,28 @@ void config_Duplicate( module_t *p_module, module_config_t *p_orig )
         p_module->p_config[i].i_type = p_orig[i].i_type;
         p_module->p_config[i].i_short = p_orig[i].i_short;
         p_module->p_config[i].i_value = p_orig[i].i_value;
+        p_module->p_config[i].i_value_orig = p_orig[i].i_value;
         p_module->p_config[i].i_min = p_orig[i].i_min;
         p_module->p_config[i].i_max = p_orig[i].i_max;
         p_module->p_config[i].f_value = p_orig[i].f_value;
+        p_module->p_config[i].f_value_orig = p_orig[i].f_value;
         p_module->p_config[i].f_min = p_orig[i].f_min;
         p_module->p_config[i].f_max = p_orig[i].f_max;
         p_module->p_config[i].b_dirty = p_orig[i].b_dirty;
+        p_module->p_config[i].b_advanced = p_orig[i].b_advanced;
 
         p_module->p_config[i].psz_type = p_orig[i].psz_type ?
-                                   strdup( _(p_orig[i].psz_type) ) : NULL;
+                                   strdup( p_orig[i].psz_type ) : NULL;
         p_module->p_config[i].psz_name = p_orig[i].psz_name ?
-                                   strdup( _(p_orig[i].psz_name) ) : NULL;
+                                   strdup( p_orig[i].psz_name ) : NULL;
         p_module->p_config[i].psz_text = p_orig[i].psz_text ?
                                    strdup( _(p_orig[i].psz_text) ) : NULL;
         p_module->p_config[i].psz_longtext = p_orig[i].psz_longtext ?
                                    strdup( _(p_orig[i].psz_longtext) ) : NULL;
         p_module->p_config[i].psz_value = p_orig[i].psz_value ?
                                    strdup( p_orig[i].psz_value ) : NULL;
+        p_module->p_config[i].psz_value_orig = p_orig[i].psz_value ?
+                                   strdup( p_orig[i].psz_value ) : NULL;
 
         p_module->p_config[i].p_lock = &p_module->object_lock;
 
@@ -456,6 +538,9 @@ void config_Free( module_t *p_module )
         if( p_item->psz_value )
             free( p_item->psz_value );
 
+        if( p_item->psz_value_orig )
+            free( p_item->psz_value_orig );
+
         if( p_item->ppsz_list )
         {
             for( i = 0; p_item->ppsz_list[i]; i++ )
@@ -499,6 +584,41 @@ void config_UnsetCallbacks( module_config_t *p_new )
     }
 }
 
+/*****************************************************************************
+ * config_ResetAll: reset the configuration data for all the modules.
+ *****************************************************************************/
+void __config_ResetAll( vlc_object_t *p_this )
+{
+    int i_index, i;
+    vlc_list_t *p_list;
+    module_t *p_module;
+
+    /* Acquire config file lock */
+    vlc_mutex_lock( &p_this->p_vlc->config_lock );
+
+    p_list = vlc_list_find( p_this, VLC_OBJECT_MODULE, FIND_ANYWHERE );
+
+    for( i_index = 0; i_index < p_list->i_count; i_index++ )
+    {
+        p_module = (module_t *)p_list->p_values[i_index].p_object ;
+        if( p_module->b_submodule ) continue;
+
+        for( i = 0; p_module->p_config[i].i_type != CONFIG_HINT_END; i++ )
+        {
+            p_module->p_config[i].i_value = p_module->p_config[i].i_value_orig;
+            p_module->p_config[i].f_value = p_module->p_config[i].f_value_orig;
+            if( p_module->p_config[i].psz_value )
+                free( p_module->p_config[i].psz_value );
+            p_module->p_config[i].psz_value =
+                p_module->p_config[i].psz_value_orig ?
+                strdup( p_module->p_config[i].psz_value_orig ) : NULL;
+        }
+    }
+
+    vlc_list_release( p_list );
+    vlc_mutex_unlock( &p_this->p_vlc->config_lock );
+}
+
 /*****************************************************************************
  * config_LoadConfigFile: loads the configuration file.
  *****************************************************************************
@@ -899,6 +1019,8 @@ int __config_SaveConfigFile( vlc_object_t *p_this, const char *psz_module_name )
                     fprintf( file, "# %s (%s)\n", p_item->psz_text,
                              (p_item->i_type == CONFIG_ITEM_BOOL) ?
                              _("boolean") : _("integer") );
+                if( p_item->i_value == p_item->i_value_orig )
+                    fprintf( file, "#" );
                 fprintf( file, "%s=%i\n", p_item->psz_name, p_item->i_value );
                 break;
 
@@ -906,6 +1028,8 @@ int __config_SaveConfigFile( vlc_object_t *p_this, const char *psz_module_name )
                 if( p_item->psz_text )
                     fprintf( file, "# %s (%s)\n", p_item->psz_text,
                              _("float") );
+                if( p_item->f_value == p_item->f_value_orig )
+                    fprintf( file, "#" );
                 fprintf( file, "%s=%f\n", p_item->psz_name,
                          (double)p_item->f_value );
                 break;
@@ -914,6 +1038,10 @@ int __config_SaveConfigFile( vlc_object_t *p_this, const char *psz_module_name )
                 if( p_item->psz_text )
                     fprintf( file, "# %s (%s)\n", p_item->psz_text,
                              _("string") );
+                if( (!p_item->psz_value && !p_item->psz_value_orig) ||
+                    (p_item->psz_value && p_item->psz_value_orig &&
+                     !strcmp( p_item->psz_value, p_item->psz_value_orig )) )
+                    fprintf( file, "#" );
                 fprintf( file, "%s=%s\n", p_item->psz_name,
                          p_item->psz_value ? p_item->psz_value : "" );
             }
@@ -1154,6 +1282,7 @@ int __config_LoadCmdLine( vlc_object_t *p_this, int *pi_argc, char *ppsz_argv[],
             {
             case CONFIG_ITEM_STRING:
             case CONFIG_ITEM_FILE:
+            case CONFIG_ITEM_DIRECTORY:
             case CONFIG_ITEM_MODULE:
                 config_PutPsz( p_this, psz_name, optarg );
                 break;
@@ -1178,6 +1307,7 @@ int __config_LoadCmdLine( vlc_object_t *p_this, int *pi_argc, char *ppsz_argv[],
             {
             case CONFIG_ITEM_STRING:
             case CONFIG_ITEM_FILE:
+            case CONFIG_ITEM_DIRECTORY:
             case CONFIG_ITEM_MODULE:
                 config_PutPsz( p_this, pp_shortopts[i_cmd]->psz_name, optarg );
                 break;
@@ -1239,7 +1369,6 @@ int __config_LoadCmdLine( vlc_object_t *p_this, int *pi_argc, char *ppsz_argv[],
 
             free( p_longopts );
             free( psz_shortopts );
-            if( b_ignore_errors ) free( ppsz_argv );
             return -1;
         }
     }
@@ -1279,7 +1408,7 @@ char *config_GetHomeDir( void )
     HINSTANCE shfolder_dll;
     SHGETFOLDERPATH SHGetFolderPath ;
 
-    /* load the shell32 dll to retreive SHGetFolderPath */
+    /* load the shfolder dll to retrieve SHGetFolderPath */
     if( ( shfolder_dll = LoadLibrary("shfolder.dll") ) != NULL )
     {
         SHGetFolderPath = (void *)GetProcAddress( shfolder_dll,
