@@ -2,7 +2,7 @@
  * modules.c : Builtin and plugin modules management functions
  *****************************************************************************
  * Copyright (C) 2001 VideoLAN
- * $Id: modules.c,v 1.113 2003/02/17 06:00:24 sam Exp $
+ * $Id: modules.c,v 1.127 2003/08/01 00:00:50 fenrir Exp $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *          Ethan C. Baldridge <BaldridgeE@cadmus.com>
@@ -67,10 +67,9 @@
 #   undef HAVE_DYNAMIC_PLUGINS
 #endif
 
-#include "error.h"
-#include "netutils.h"
+#include "vlc_error.h"
 
-#include "interface.h"
+#include "vlc_interface.h"
 #include "vlc_playlist.h"
 #include "intf_eject.h"
 
@@ -78,14 +77,18 @@
 #include "input_ext-intf.h"
 #include "input_ext-dec.h"
 #include "input_ext-plugins.h"
+#include "ninput.h"
 
-#include "video.h"
+#include "vlc_video.h"
 #include "video_output.h"
+#include "vout_synchro.h"
 
 #include "audio_output.h"
 #include "aout_internal.h"
 
 #include "stream_output.h"
+#include "announce.h"
+#include "osd.h"
 
 #include "iso_lang.h"
 
@@ -340,23 +343,40 @@ module_t * __module_Need( vlc_object_t *p_this, const char *psz_capability,
         /* If we required a shortcut, check this plugin provides it. */
         if( i_shortcuts )
         {
-            vlc_bool_t b_trash = VLC_TRUE;
+            vlc_bool_t b_trash;
             int i_dummy, i_short = i_shortcuts;
             char *psz_name = psz_shortcuts;
 
+            /* Let's drop modules with a 0 score (unless they are
+             * explicitly requested) */
+            b_trash = !p_module->i_score;
+
             while( i_short )
             {
-                for( i_dummy = 0;
-                     b_trash && p_module->pp_shortcuts[i_dummy];
-                     i_dummy++ )
+                /* If the last given shortcut is "none" and we couldn't
+                 * find the module in the list of provided shortcuts,
+                 * then kick the bastard out of here!!! */
+                if( (i_short == 1) && !strcmp(psz_name, "none") )
                 {
-                    b_trash = ( strcmp(psz_name, "any") || !p_module->i_score )
-                        && strcmp( psz_name, p_module->pp_shortcuts[i_dummy] );
+                    b_trash = VLC_TRUE;
+                    break;
                 }
 
-                if( !b_trash )
+                for( i_dummy = 0; p_module->pp_shortcuts[i_dummy]; i_dummy++ )
                 {
-                    i_shortcut_bonus = i_short * 10000;
+                    if( !strcmp( psz_name,
+                                 p_module->pp_shortcuts[i_dummy] ) )
+                    {
+                        /* Found it */
+                        b_trash = VLC_FALSE;
+                        i_shortcut_bonus = i_short * 10000;
+                        break;
+                    }
+                }
+
+                if( i_shortcut_bonus )
+                {
+                    /* We found it... remember ? */
                     break;
                 }
 
@@ -381,7 +401,7 @@ module_t * __module_Need( vlc_object_t *p_this, const char *psz_capability,
         }
 
         /* Special case: test if we requested a particular intf plugin */
-        if( p_module->psz_program
+        if( !i_shortcuts && p_module->psz_program
              && !strcmp( p_module->psz_program,
                          p_this->p_vlc->psz_object_name ) )
         {
@@ -565,6 +585,16 @@ static void AllocateAllPlugins( vlc_object_t *p_this )
     path[ sizeof(path)/sizeof(char*) - 2 ] = config_GetPsz( p_this,
                                                             "plugin-path" );
 
+#if defined( WIN32 ) && !defined( UNDER_CE ) && !defined( _MSC_VER )
+    /* If there is no 'plugins' nor 'modules' subdirectory, the user may have
+     * screwed up the unzipping stage, so we look into '.' instead */
+    if( !opendir( "plugins" ) && !opendir( "modules" )
+        && !strcmp( *ppsz_path, "modules" ) )
+    {
+        *ppsz_path = ".";
+    }
+#endif
+
     for( ; *ppsz_path != NULL ; ppsz_path++ )
     {
 #if defined( SYS_BEOS ) || defined( SYS_DARWIN ) \
@@ -576,9 +606,9 @@ static void AllocateAllPlugins( vlc_object_t *p_this )
         /* Under BeOS, we need to add beos_GetProgramPath() to access
          * files under the current directory */
 #ifdef WIN32
-        if( i_dirlen < 3 || ppsz_path[3] != '\\' )
+        if( i_dirlen < 3 || (*ppsz_path)[3] != '\\' )
 #else
-        if( ppsz_path[0] != '/' )
+        if( (*ppsz_path)[0] != '/' )
 #endif
         {
             i_dirlen += i_vlclen + 2;
@@ -622,7 +652,8 @@ static void AllocateAllPlugins( vlc_object_t *p_this )
     }
 
     /* Free plugin-path */
-    free( path[ sizeof(path)/sizeof(char*) - 2 ] );
+    if( path[ sizeof(path)/sizeof(char*) - 2 ] )
+        free( path[ sizeof(path)/sizeof(char*) - 2 ] );
     path[ sizeof(path)/sizeof(char*) - 2 ] = NULL;
 }
 
@@ -632,8 +663,12 @@ static void AllocateAllPlugins( vlc_object_t *p_this )
 static void AllocatePluginDir( vlc_object_t *p_this, const MYCHAR *psz_dir,
                                int i_maxdepth )
 {
+#if defined( UNDER_CE ) || defined( _MSC_VER )
 #ifdef UNDER_CE
     MYCHAR psz_path[MAX_PATH + 256];
+#else
+    char psz_path[MAX_PATH + 256];
+#endif
     WIN32_FIND_DATA finddata;
     HANDLE handle;
     unsigned int rc;
@@ -649,7 +684,7 @@ static void AllocatePluginDir( vlc_object_t *p_this, const MYCHAR *psz_dir,
         return;
     }
 
-#ifdef UNDER_CE
+#if defined( UNDER_CE ) || defined( _MSC_VER )
     rc = GetFileAttributes( psz_dir );
     if( !(rc & FILE_ATTRIBUTE_DIRECTORY) )
     {
@@ -658,7 +693,11 @@ static void AllocatePluginDir( vlc_object_t *p_this, const MYCHAR *psz_dir,
     }
 
     /* Parse all files in the directory */
+#ifdef UNDER_CE
     swprintf( psz_path, L"%s\\*.*", psz_dir );
+#else
+    sprintf( psz_path, "%s\\*.*", psz_dir );
+#endif
     handle = FindFirstFile( psz_path, &finddata );
     if( handle == INVALID_HANDLE_VALUE )
     {
@@ -669,15 +708,33 @@ static void AllocatePluginDir( vlc_object_t *p_this, const MYCHAR *psz_dir,
     /* Parse the directory and try to load all files it contains. */
     do
     {
+#ifdef UNDER_CE
         unsigned int i_len = wcslen( finddata.cFileName );
-
         swprintf( psz_path, L"%s\\%s", psz_dir, finddata.cFileName );
+#else
+        unsigned int i_len = strlen( finddata.cFileName );
+        /* Skip ".", ".." and anything starting with "." */
+        if( !*finddata.cFileName || *finddata.cFileName == '.' )
+        {
+            if( !FindNextFile( handle, &finddata ) ) break;
+            continue;
+        }
+        sprintf( psz_path, "%s\\%s", psz_dir, finddata.cFileName );
+#endif
 
         if( GetFileAttributes( psz_path ) & FILE_ATTRIBUTE_DIRECTORY )
         {
             AllocatePluginDir( p_this, psz_path, i_maxdepth - 1 );
         }
-        else if( i_len > strlen( LIBEXT ) )
+        else if( i_len > strlen( LIBEXT )
+#ifdef UNDER_CE
+                )
+#else
+                  /* We only load files ending with LIBEXT */
+                  && !strncasecmp( psz_path + strlen( psz_path)
+                                   - strlen( LIBEXT ),
+                                   LIBEXT, strlen( LIBEXT ) ) )
+#endif
         {
             AllocatePluginFile( p_this, psz_path );
         }
