@@ -2,7 +2,7 @@
  * playlist.c : Playlist management functions
  *****************************************************************************
  * Copyright (C) 1999-2001 VideoLAN
- * $Id: playlist.c,v 1.34 2003/03/18 00:49:14 gbazin Exp $
+ * $Id: playlist.c,v 1.43 2003/07/23 01:13:48 gbazin Exp $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -32,11 +32,6 @@
 #include "vlc_playlist.h"
 
 #define PLAYLIST_FILE_HEADER_0_5  "# vlc playlist file version 0.5"
-#ifdef WIN32
-#   define PLAYLIST_FILE_EOL "\r\n"
-#else
-#   define PLAYLIST_FILE_EOL "\n"
-#endif
 
 /*****************************************************************************
  * Local prototypes
@@ -69,6 +64,12 @@ playlist_t * __playlist_Create ( vlc_object_t *p_parent )
     val.b_bool = VLC_TRUE;
     var_Set( p_playlist, "intf-change", val );
 
+    var_Create( p_playlist, "intf-popupmenu", VLC_VAR_VOID );
+
+    var_Create( p_playlist, "intf-show", VLC_VAR_BOOL );
+    val.b_bool = VLC_TRUE;
+    var_Set( p_playlist, "intf-show", val );
+
     p_playlist->p_input = NULL;
     p_playlist->i_status = PLAYLIST_STOPPED;
     p_playlist->i_index = -1;
@@ -111,8 +112,24 @@ void playlist_Destroy( playlist_t * p_playlist )
  * Add an item to the playlist at position i_pos. If i_pos is PLAYLIST_END,
  * add it at the end regardless of the playlist current size.
  *****************************************************************************/
-int playlist_Add( playlist_t *p_playlist, const char * psz_target,
-                                          int i_mode, int i_pos )
+int playlist_Add( playlist_t *p_playlist, const char *psz_target,
+                  const char **ppsz_options, int i_options,
+                  int i_mode, int i_pos )
+{
+    return playlist_AddName( p_playlist, psz_target, psz_target,
+                             ppsz_options, i_options, i_mode, i_pos );
+}
+
+/*****************************************************************************
+ * playlist_AddName: add an item to the playlist with his name
+ *****************************************************************************
+ * Add an item to the playlist at position i_pos. If i_pos is PLAYLIST_END,
+ * add it at the end regardless of the playlist current size.
+ *****************************************************************************/
+int playlist_AddName( playlist_t *p_playlist, const char *psz_name, 
+                      const char *psz_uri,
+                      const char **ppsz_options, int i_options,
+                      int i_mode, int i_pos )
 {
     playlist_item_t * p_item;
 
@@ -122,16 +139,28 @@ int playlist_Add( playlist_t *p_playlist, const char * psz_target,
         msg_Err( p_playlist, "out of memory" );
     }
 
-    p_item->psz_name = strdup( psz_target );
-    p_item->psz_uri  = strdup( psz_target );
+    p_item->psz_name = strdup( psz_name );
+    p_item->psz_uri  = strdup( psz_uri );
     p_item->i_type = 0;
     p_item->i_status = 0;
     p_item->b_autodeletion = VLC_FALSE;
 
+    p_item->ppsz_options = NULL;
+    p_item->i_options = i_options;
+
+    if( i_options )
+    {
+        int i;
+
+        p_item->ppsz_options = (char **)malloc( i_options * sizeof(char *) );
+        for( i = 0; i < i_options; i++ )
+            p_item->ppsz_options[i] = strdup( ppsz_options[i] );
+
+    }
+
     return playlist_AddItem( p_playlist, p_item, i_mode, i_pos );
 }
 
-
 int playlist_AddItem( playlist_t *p_playlist, playlist_item_t * p_item,
                 int i_mode, int i_pos)
 {
@@ -172,7 +201,7 @@ int playlist_AddItem( playlist_t *p_playlist, playlist_item_t * p_item,
     }
 
 
-    msg_Dbg( p_playlist, "adding playlist item  %s ", p_item->psz_name );
+    msg_Dbg( p_playlist, "adding playlist item  %s  ( %s )", p_item->psz_name, p_item->psz_uri);
 
     /* Create the new playlist item */
 
@@ -274,6 +303,15 @@ int playlist_Delete( playlist_t * p_playlist, int i_pos )
         {
             free( p_playlist->pp_items[i_pos]->psz_uri );
         }
+        if( p_playlist->pp_items[i_pos]->i_options )
+        {
+            int i;
+
+            for( i = 0; i < p_playlist->pp_items[i_pos]->i_options; i++ )
+                free( p_playlist->pp_items[i_pos]->ppsz_options[i] );
+
+            free( p_playlist->pp_items[i_pos]->ppsz_options );
+        }
 
         /* XXX: what if the item is still in use? */
         free( p_playlist->pp_items[i_pos] );
@@ -383,6 +421,10 @@ void playlist_Command( playlist_t * p_playlist, int i_command, int i_arg )
 
     case PLAYLIST_PLAY:
         p_playlist->i_status = PLAYLIST_RUNNING;
+        if( !p_playlist->p_input )
+        {
+            PlayItem( p_playlist );
+        }
         if( p_playlist->p_input )
         {
             input_SetStatus( p_playlist->p_input, INPUT_STATUS_PLAY );
@@ -494,6 +536,7 @@ static void RunThread ( playlist_t *p_playlist )
         }
         else if( p_playlist->i_status != PLAYLIST_STOPPED )
         {
+            SkipItem( p_playlist, 0 );
             PlayItem( p_playlist );
         }
 
@@ -560,6 +603,7 @@ static void SkipItem( playlist_t *p_playlist, int i_arg )
 {
     int i_oldindex = p_playlist->i_index;
     vlc_bool_t b_random;
+    vlc_value_t val;
 
     /* If the playlist is empty, there is no current item */
     if( p_playlist->i_size == 0 )
@@ -607,6 +651,9 @@ static void SkipItem( playlist_t *p_playlist, int i_arg )
     {
         p_playlist->i_index = p_playlist->i_size - 1;
     }
+
+    val.b_bool = VLC_TRUE;
+    var_Set( p_playlist, "intf-change", val );
 }
 
 /*****************************************************************************
@@ -713,7 +760,8 @@ int playlist_LoadFile( playlist_t * p_playlist, const char *psz_filename )
            if( line[strlen(line)-1] == '\r' ) line[strlen(line)-1] = (char)0;
        }
 
-       playlist_Add ( p_playlist , (char*) &line , PLAYLIST_APPEND , PLAYLIST_END );
+       playlist_Add ( p_playlist , (char *)&line ,
+                      0, 0, PLAYLIST_APPEND , PLAYLIST_END );
     }
 
     /* start playing */
@@ -747,12 +795,12 @@ int playlist_SaveFile( playlist_t * p_playlist, const char * psz_filename )
         return -1;
     }
 
-    fprintf( file , PLAYLIST_FILE_HEADER_0_5 PLAYLIST_FILE_EOL );
+    fprintf( file , PLAYLIST_FILE_HEADER_0_5 "\n" );
 
     for ( i = 0 ; i < p_playlist->i_size ; i++ )
     {
         fprintf( file , p_playlist->pp_items[i]->psz_uri );
-        fprintf( file , PLAYLIST_FILE_EOL );
+        fprintf( file , "\n" );
     }
 
     fclose( file );
