@@ -2,7 +2,7 @@
  * vout_subpictures.c : subpicture management functions
  *****************************************************************************
  * Copyright (C) 2000 VideoLAN
- * $Id: vout_subpictures.c,v 1.13 2002/04/25 21:52:42 sam Exp $
+ * $Id: vout_subpictures.c,v 1.21 2003/07/15 18:12:05 sigmunau Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
@@ -25,22 +25,23 @@
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
-#include <errno.h>                                                 /* ENOMEM */
 #include <stdlib.h>                                                /* free() */
 #include <stdio.h>                                              /* sprintf() */
 #include <string.h>                                            /* strerror() */
 
-#include <videolan/vlc.h>
+#include <vlc/vlc.h>
 
-#include "video.h"
+#include "vlc_video.h"
 #include "video_output.h"
 
-/*****************************************************************************
- * vout_DisplaySubPicture: display a subpicture unit
- *****************************************************************************
+/**
+ * Display a subpicture unit
+ *
  * Remove the reservation flag of a subpicture, which will cause it to be
  * ready for display.
- *****************************************************************************/
+ * \param p_vout the video output this subpicture should be displayed on
+ * \param p_subpic the subpicture to display
+ */
 void  vout_DisplaySubPicture( vout_thread_t *p_vout, subpicture_t *p_subpic )
 {
     int         i_margin;
@@ -48,17 +49,18 @@ void  vout_DisplaySubPicture( vout_thread_t *p_vout, subpicture_t *p_subpic )
     /* Check if status is valid */
     if( p_subpic->i_status != RESERVED_SUBPICTURE )
     {
-        intf_ErrMsg( "vout error: subpicture %p has invalid status #%d",
-                     p_subpic, p_subpic->i_status );
+        msg_Err( p_vout, "subpicture %p has invalid status #%d",
+                         p_subpic, p_subpic->i_status );
     }
 
     /* If the user requested an SPU margin, we force the position after
      * having checked that it was a valid value. */
-    i_margin = config_GetIntVariable( "spumargin" );
+    i_margin = config_GetInt( p_vout, "spumargin" );
 
     if( i_margin >= 0 )
     {
-        if( p_subpic->i_height + i_margin <= p_vout->output.i_height )
+        if( p_subpic->i_height + (unsigned int)i_margin
+                                                 <= p_vout->output.i_height )
         {
             p_subpic->i_y = p_vout->output.i_height
                              - i_margin - p_subpic->i_height;
@@ -69,20 +71,21 @@ void  vout_DisplaySubPicture( vout_thread_t *p_vout, subpicture_t *p_subpic )
     p_subpic->i_status = READY_SUBPICTURE;
 }
 
-/*****************************************************************************
- * vout_CreateSubPicture: allocate a subpicture in the video output heap.
- *****************************************************************************
+/**
+ * Allocate a subpicture in the video output heap.
+ *
  * This function create a reserved subpicture in the video output heap.
  * A null pointer is returned if the function fails. This method provides an
  * already allocated zone of memory in the spu data fields. It needs locking
  * since several pictures can be created by several producers threads.
- *****************************************************************************/
-subpicture_t *vout_CreateSubPicture( vout_thread_t *p_vout, int i_type,
-                                     int i_size )
+ * \param p_vout the vout in which to create the subpicture
+ * \param i_type the type of the subpicture
+ * \return NULL on error, a reserved subpicture otherwise
+ */
+subpicture_t *vout_CreateSubPicture( vout_thread_t *p_vout, int i_type )
 {
     int                 i_subpic;                        /* subpicture index */
-    subpicture_t *      p_free_subpic = NULL;       /* first free subpicture */
-    subpicture_t *      p_destroyed_subpic = NULL; /* first destroyed subpic */
+    subpicture_t *      p_subpic = NULL;            /* first free subpicture */
 
     /* Get lock */
     vlc_mutex_lock( &p_vout->subpicture_lock );
@@ -92,100 +95,77 @@ subpicture_t *vout_CreateSubPicture( vout_thread_t *p_vout, int i_type,
      */
     for( i_subpic = 0; i_subpic < VOUT_MAX_SUBPICTURES; i_subpic++ )
     {
-        if( p_vout->p_subpicture[i_subpic].i_status == DESTROYED_SUBPICTURE )
-        {
-            /* Subpicture is marked for destruction, but is still allocated */
-            if( (p_vout->p_subpicture[i_subpic].i_type  == i_type)   &&
-                (p_vout->p_subpicture[i_subpic].i_size  >= i_size) )
-            {
-                /* Memory size do match or is smaller : memory will not be
-                 * reallocated, and function can end immediately - this is
-                 * the best possible case, since no memory allocation needs
-                 * to be done */
-                p_vout->p_subpicture[i_subpic].i_status = RESERVED_SUBPICTURE;
-                vlc_mutex_unlock( &p_vout->subpicture_lock );
-                return( &p_vout->p_subpicture[i_subpic] );
-            }
-            else if( p_destroyed_subpic == NULL )
-            {
-                /* Memory size do not match, but subpicture index will be kept
-                 * in case we find no other place */
-                p_destroyed_subpic = &p_vout->p_subpicture[i_subpic];
-            }
-        }
-        else if( (p_free_subpic == NULL) &&
-                 (p_vout->p_subpicture[i_subpic].i_status == FREE_SUBPICTURE ))
+        if( p_vout->p_subpicture[i_subpic].i_status == FREE_SUBPICTURE )
         {
             /* Subpicture is empty and ready for allocation */
-            p_free_subpic = &p_vout->p_subpicture[i_subpic];
+            p_subpic = &p_vout->p_subpicture[i_subpic];
+            p_vout->p_subpicture[i_subpic].i_status = RESERVED_SUBPICTURE;
+            break;
         }
     }
 
-    /* If no free subpictures are available, use a destroyed subpicture */
-    if( (p_free_subpic == NULL) && (p_destroyed_subpic != NULL ) )
+    /* If no free subpicture could be found */
+    if( p_subpic == NULL )
     {
-        /* No free subpicture or matching destroyed subpictures have been
-         * found, but a destroyed subpicture is still available */
-        free( p_destroyed_subpic->p_sys_orig );
-        p_free_subpic = p_destroyed_subpic;
-    }
-
-    /* If no free or destroyed subpicture could be found */
-    if( p_free_subpic == NULL )
-    {
-        intf_ErrMsg( "vout error: subpicture heap is full" );
+        msg_Err( p_vout, "subpicture heap is full" );
         vlc_mutex_unlock( &p_vout->subpicture_lock );
-        return( NULL );
+        return NULL;
     }
 
-    p_free_subpic->p_sys =
-        vlc_memalign( &p_free_subpic->p_sys_orig, 16, i_size );
+    /* Copy subpicture information, set some default values */
+    p_subpic->i_type    = i_type;
+    p_subpic->i_status  = RESERVED_SUBPICTURE;
 
-    if( p_free_subpic->p_sys != NULL )
-    {
-        /* Copy subpicture information, set some default values */
-        p_free_subpic->i_type   = i_type;
-        p_free_subpic->i_status = RESERVED_SUBPICTURE;
-        p_free_subpic->i_size   = i_size;
-        p_free_subpic->i_x      = 0;
-        p_free_subpic->i_y      = 0;
-        p_free_subpic->i_width  = 0;
-        p_free_subpic->i_height = 0;
-    }
-    else
-    {
-        /* Memory allocation failed : set subpicture as empty */
-        p_free_subpic->i_type   = EMPTY_SUBPICTURE;
-        p_free_subpic->i_status = FREE_SUBPICTURE;
-        p_free_subpic           = NULL;
-        intf_ErrMsg( "vout error: spu allocation returned %s",
-                     strerror( ENOMEM ) );
-    }
+    p_subpic->i_start   = 0;
+    p_subpic->i_stop    = 0;
+    p_subpic->b_ephemer = VLC_FALSE;
+
+    p_subpic->i_x       = 0;
+    p_subpic->i_y       = 0;
+    p_subpic->i_width   = 0;
+    p_subpic->i_height  = 0;
 
     vlc_mutex_unlock( &p_vout->subpicture_lock );
 
-    return( p_free_subpic );
+    return p_subpic;
 }
 
-/*****************************************************************************
- * vout_DestroySubPicture: remove a subpicture from the heap
- *****************************************************************************
+/**
+ * Remove a subpicture from the heap
+ *
  * This function frees a previously reserved subpicture.
  * It is meant to be used when the construction of a picture aborted.
  * This function does not need locking since reserved subpictures are ignored
  * by the output thread.
- *****************************************************************************/
+ */
 void vout_DestroySubPicture( vout_thread_t *p_vout, subpicture_t *p_subpic )
 {
-   /* Check if status is valid */
-   if( ( p_subpic->i_status != RESERVED_SUBPICTURE )
-          && ( p_subpic->i_status != READY_SUBPICTURE ) )
-   {
-       intf_ErrMsg( "vout error: subpicture %p has invalid status %d",
-                   p_subpic, p_subpic->i_status );
-   }
+    /* Get lock */
+    vlc_mutex_lock( &p_vout->subpicture_lock );
 
-    p_subpic->i_status = DESTROYED_SUBPICTURE;
+    /* There can be race conditions so we need to check the status */
+    if( p_subpic->i_status == FREE_SUBPICTURE )
+    {
+        vlc_mutex_unlock( &p_vout->subpicture_lock );
+        return;
+    }
+
+    /* Check if status is valid */
+    if( ( p_subpic->i_status != RESERVED_SUBPICTURE )
+           && ( p_subpic->i_status != READY_SUBPICTURE ) )
+    {
+        msg_Err( p_vout, "subpicture %p has invalid status %d",
+                         p_subpic, p_subpic->i_status );
+    }
+
+    if( p_subpic->pf_destroy )
+    {
+        p_subpic->pf_destroy( p_subpic );
+    }
+
+    p_subpic->i_status = FREE_SUBPICTURE;
+
+    vlc_mutex_unlock( &p_vout->subpicture_lock );
 }
 
 /*****************************************************************************
@@ -196,11 +176,17 @@ void vout_DestroySubPicture( vout_thread_t *p_vout, subpicture_t *p_subpic )
 void vout_RenderSubPictures( vout_thread_t *p_vout, picture_t *p_pic,
                              subpicture_t *p_subpic )
 {
-    while( p_subpic != NULL )
+    /* Get lock */
+    vlc_mutex_lock( &p_vout->subpicture_lock );
+
+    /* Check i_status again to make sure spudec hasn't destroyed the subpic */
+    while( p_subpic != NULL && p_subpic->i_status != FREE_SUBPICTURE )
     {
         p_subpic->pf_render( p_vout, p_pic, p_subpic );
         p_subpic = p_subpic->p_next;
     }
+
+    vlc_mutex_unlock( &p_vout->subpicture_lock );
 }
 
 /*****************************************************************************
@@ -231,7 +217,8 @@ subpicture_t *vout_SortSubPictures( vout_thread_t *p_vout,
             /* If it is a DVD subpicture, check its date */
             if( p_vout->p_subpicture[i_index].i_type == MEMORY_SUBPICTURE )
             {
-                if( display_date > p_vout->p_subpicture[i_index].i_stop )
+                if( !p_vout->p_subpicture[i_index].b_ephemer
+                     && display_date > p_vout->p_subpicture[i_index].i_stop )
                 {
                     /* Too late, destroy the subpic */
                     vout_DestroySubPicture( p_vout,
@@ -239,7 +226,8 @@ subpicture_t *vout_SortSubPictures( vout_thread_t *p_vout,
                     continue;
                 }
 
-                if( display_date < p_vout->p_subpicture[i_index].i_start )
+                if( display_date
+                     && display_date < p_vout->p_subpicture[i_index].i_start )
                 {
                     /* Too early, come back next monday */
                     continue;
